@@ -17,17 +17,15 @@
 
 #include <config.h>
 
-#ifdef HAVE_UNISTD_H
-#    include <unistd.h>
-#endif
-#include <sys/types.h>
 #include <signal.h>
 #include <stdio.h>
 
+#include <sys/types.h>
 #ifdef __FreeBSD__
-#include <sys/consio.h>
-#include <sys/ioctl.h>
-#endif				/* __FreeBSD__ */
+#  include <sys/consio.h>
+#  include <sys/ioctl.h>
+#endif
+#include <unistd.h>
 
 #include "global.h"
 #include "tty.h"
@@ -144,14 +142,12 @@ handle_console_linux (unsigned char action)
 	    open ("/dev/null", O_WRONLY);
 	    if (tty_name) {
 		/* Exec the console save/restore handler */
-		mc_conssaver = concat_dir_and_file (LIBDIR, "cons.saver");
+		mc_conssaver = concat_dir_and_file (SAVERDIR, "cons.saver");
 		execl (mc_conssaver, "cons.saver", tty_name, (char *) NULL);
 	    }
 	    /* Console is not a tty or execl() failed */
 	    console_flag = 0;
 	    write (1, &console_flag, 1);
-	    close (1);
-	    close (0);
 	    _exit (3);
 	}			/* if (cons_saver_pid ...) */
 	break;
@@ -382,6 +378,8 @@ show_console_contents (int starty, unsigned char begin_line,
 void
 handle_console (unsigned char action)
 {
+    (void) action;
+
     if (look_for_rxvt_extensions ())
 	return;
 
