@@ -17,10 +17,12 @@
  */
 
 #include <config.h>
-#include <string.h>
-#include <stdio.h>
+
 #include <ctype.h>
 #include <stdarg.h>
+#include <stdio.h>
+#include <string.h>
+
 #include "global.h"
 #include "tty.h"
 #include "mouse.h"
@@ -133,6 +135,8 @@ init_widget (Widget *w, int y, int x, int lines, int cols,
 cb_ret_t
 default_proc (widget_msg_t msg, int parm)
 {
+    (void) parm;
+
     switch (msg) {
     case WIDGET_INIT:
     case WIDGET_FOCUS:
@@ -156,12 +160,12 @@ common_dialog_repaint (struct Dlg_head *h)
 
     space = (h->flags & DLG_COMPACT) ? 0 : 1;
 
-    attrset (NORMALC);
+    attrset (DLG_NORMALC (h));
     dlg_erase (h);
     draw_box (h, space, space, h->lines - 2 * space, h->cols - 2 * space);
 
     if (h->title) {
-	attrset (HOT_NORMALC);
+	attrset (DLG_HOT_NORMALC (h));
 	dlg_move (h, space, (h->cols - strlen (h->title)) / 2);
 	addstr (h->title);
     }
@@ -170,6 +174,8 @@ common_dialog_repaint (struct Dlg_head *h)
 /* Default dialog callback */
 cb_ret_t default_dlg_callback (Dlg_head *h, dlg_msg_t msg, int parm)
 {
+    (void) parm;
+
     if (msg == DLG_DRAW && h->color) {
 	common_dialog_repaint (h);
 	return MSG_HANDLED;
@@ -198,7 +204,7 @@ create_dlg (int y1, int x1, int lines, int cols, const int *color_set,
 
     new_d = g_new0 (Dlg_head, 1);
     new_d->color = color_set;
-    new_d->help_ctx = const_cast(char *, help_ctx);
+    new_d->help_ctx = help_ctx;
     new_d->callback = callback ? callback : default_dlg_callback;
     new_d->x = x1;
     new_d->y = y1;
@@ -288,7 +294,8 @@ do_refresh (void)
 }
 
 /* broadcast a message to all the widgets in a dialog that have
- * the options set to flags.
+ * the options set to flags. If flags is zero, the message is sent
+ * to all widgets.
  */
 static void
 dlg_broadcast_msg_to (Dlg_head *h, widget_msg_t message, int reverse,
@@ -310,7 +317,8 @@ dlg_broadcast_msg_to (Dlg_head *h, widget_msg_t message, int reverse,
 	    p = p->prev;
 	else
 	    p = p->next;
-	send_message (wi, message, 0);
+	if (flags == 0 || (flags & wi->options))
+	    send_message (wi, message, 0);
     } while (first != p);
 }
 
@@ -318,7 +326,7 @@ dlg_broadcast_msg_to (Dlg_head *h, widget_msg_t message, int reverse,
 void
 dlg_broadcast_msg (Dlg_head *h, widget_msg_t message, int reverse)
 {
-    dlg_broadcast_msg_to (h, message, reverse, ~0);
+    dlg_broadcast_msg_to (h, message, reverse, 0);
 }
 
 int dlg_focus (Dlg_head *h)
@@ -403,7 +411,7 @@ dlg_select_by_id (Dlg_head *h, int id)
     } while (w != h->current);
 
     if (w_found)
-	dlg_select_widget(h, w_found);
+	dlg_select_widget(w_found);
 }
 
 
@@ -456,9 +464,9 @@ do_select_widget (Dlg_head *h, Widget *w, select_dir_t dir)
  * Try to select widget in the dialog.
  */
 void
-dlg_select_widget (Dlg_head *h, void *w)
+dlg_select_widget (void *w)
 {
-    do_select_widget (h, w, SELECT_NEXT);
+    do_select_widget (((Widget *) w)->parent, w, SELECT_NEXT);
 }
 
 
@@ -819,12 +827,14 @@ void widget_set_size (Widget *widget, int y, int x, int lines, int cols)
     widget->y = y;
     widget->cols = cols;
     widget->lines = lines;
+    send_message (widget, WIDGET_RESIZED, 0 /* unused */);
 }
 
-/* Replace widget old_w for widget new_w in the dialog h */
+/* Replace widget old_w for widget new_w in the dialog */
 void
-dlg_replace_widget (Dlg_head *h, Widget *old_w, Widget *new_w)
+dlg_replace_widget (Widget *old_w, Widget *new_w)
 {
+    Dlg_head *h = old_w->parent;
     int should_focus = 0;
 
     if (!h->current)
@@ -854,7 +864,7 @@ dlg_replace_widget (Dlg_head *h, Widget *old_w, Widget *new_w)
     send_message (new_w, WIDGET_INIT, 0);
 
     if (should_focus)
-	dlg_select_widget (h, new_w);
+	dlg_select_widget (new_w);
 
     send_message (new_w, WIDGET_DRAW, 0);
 }
