@@ -39,18 +39,15 @@
 /* {{{ Include files */
 
 #include <config.h>
-/* Hack: the vfs code should not rely on this */
-#define WITH_FULL_PATHS 1
+
+#include <ctype.h>
+#include <errno.h>
+#include <stdio.h>
+#include <string.h>
 
 #include <sys/types.h>
-#include <stdio.h>
-#include <errno.h>
-#include <ctype.h>
-#include <string.h>
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif				/* HAVE_UNISTD_H */
 #include <sys/stat.h>
+#include <unistd.h>
 
 #include "global.h"
 #include "tty.h"
@@ -60,8 +57,7 @@
 #include "win.h"
 #include "dialog.h"
 #include "widget.h"
-#define WANT_WIDGETS
-#include "main.h"		/* WANT_WIDGETS-> we get the the_hint def */
+#include "main.h"		/* cmd_buf */
 #include "layout.h"
 #include "widget.h"
 #include "wtools.h"
@@ -78,6 +74,9 @@
 
 /* }}} */
 
+/* Hack: the vfs code should not rely on this */
+#define WITH_FULL_PATHS 1
+
 int verbose = 1;
 
 /*
@@ -87,9 +86,6 @@ int verbose = 1;
  */
 int file_op_compute_totals = 1;
 
-/* If on, default for "No" in delete operations */
-int safe_delete = 0;
-
 /* This is a hard link cache */
 struct link {
     struct link *next;
@@ -97,7 +93,7 @@ struct link {
     dev_t dev;
     ino_t ino;
     short linkcount;
-    umode_t st_mode;
+    mode_t st_mode;
     char name[1];
 };
 
@@ -142,33 +138,33 @@ static int files_error (const char *format, const char *file1,
 enum CaseConvs { NO_CONV = 0, UP_CHAR = 1, LOW_CHAR = 2, UP_SECT =
 	4, LOW_SECT = 8 };
 
-static int
-convert_case (int c, enum CaseConvs *conversion)
+static char
+convert_case (char c, enum CaseConvs *conversion)
 {
     if (*conversion & UP_CHAR) {
 	*conversion &= ~UP_CHAR;
-	return toupper (c);
+	return toupper ((unsigned char) c);
     } else if (*conversion & LOW_CHAR) {
 	*conversion &= ~LOW_CHAR;
-	return tolower (c);
+	return tolower ((unsigned char) c);
     } else if (*conversion & UP_SECT) {
-	return toupper (c);
+	return toupper ((unsigned char) c);
     } else if (*conversion & LOW_SECT) {
-	return tolower (c);
+	return tolower ((unsigned char) c);
     } else
 	return c;
 }
 
 static int transform_error = 0;
 
-static const unsigned char *
-do_transform_source (FileOpContext *ctx, const unsigned char *source)
+static const char *
+do_transform_source (FileOpContext *ctx, const char *source)
 {
     size_t j, k, l, len;
-    unsigned const char *fnsource = x_basename (source);
+    const char *fnsource = x_basename (source);
     int next_reg;
     enum CaseConvs case_conv = NO_CONV;
-    static unsigned char fntarget[MC_MAXPATHLEN];
+    static char fntarget[MC_MAXPATHLEN];
 
     len = strlen (fnsource);
     j = re_match (&ctx->rx, fnsource, len, 0, &ctx->regs);
@@ -234,12 +230,12 @@ do_transform_source (FileOpContext *ctx, const unsigned char *source)
     return fntarget;
 }
 
-static const unsigned char *
-transform_source (FileOpContext *ctx, const unsigned char *source)
+static const char *
+transform_source (FileOpContext *ctx, const char *source)
 {
-    unsigned char *s = g_strdup (source);
-    unsigned char *q;
-    const unsigned char *p;
+    char *s = g_strdup (source);
+    char *q;
+    const char *p;
 
     /* We remove \n from the filename since regex routines would use \n as an anchor */
     /* this is just to be allowed to maniupulate file names with \n on it */
@@ -382,7 +378,7 @@ make_symlink (FileOpContext *ctx, const char *src_path, const char *dst_path)
 	const char *r = strrchr (src_path, PATH_SEP);
 
 	if (r) {
-	    p = g_strndup (src_path, r - src_path);
+	    p = g_strndup (src_path, r - src_path + 1);
 	    if (*dst_path == PATH_SEP)
 		q = g_strdup (dst_path);
 	    else
@@ -533,7 +529,7 @@ copy_file_file (FileOpContext *ctx, const char *src_path, const char *dst_path,
 	if (!ctx->follow_links && sb.st_nlink > 1 &&
 	    check_hardlinks (src_path, dst_path, &sb) == 1) {
 	    /* We have made a hardlink - no more processing is necessary */
-	    return return_status;
+	    return FILE_CONT;
 	}
 
 	if (S_ISLNK (sb.st_mode)) {
@@ -738,10 +734,10 @@ copy_file_file (FileOpContext *ctx, const char *src_path, const char *dst_path,
 
 	    file_progress_set_stalled_label (ctx, stalled_msg);
 	    return_status = file_progress_show_bytes (ctx, *progress_bytes +
-		    n_read_total, ctx->progress_bytes);
+		    n_read_total + ctx->do_reget, ctx->progress_bytes);
 	    if (return_status == FILE_CONT) {
 		return_status =
-		    file_progress_show (ctx, n_read_total, file_size);
+		    file_progress_show (ctx, n_read_total + ctx->do_reget, file_size);
 	    }
 	    mc_refresh ();
 	    if (return_status != FILE_CONT)
@@ -794,7 +790,7 @@ copy_file_file (FileOpContext *ctx, const char *src_path, const char *dst_path,
 	    }
 	}
 
-	if (!appending && ctx->preserve) {
+	if (!appending) {
 	    while (mc_chmod (dst_path, (src_mode & ctx->umask_kill))) {
 		temp_status = file_error (
 			_(" Cannot chmod target file \"%s\" \n %s "), dst_path);
@@ -1026,6 +1022,7 @@ move_file_file (FileOpContext *ctx, const char *s, const char *d,
 {
     struct stat src_stats, dst_stats;
     int return_status = FILE_CONT;
+    gboolean copy_done = FALSE;
 
     if (file_progress_show_source (ctx, s) == FILE_ABORT
 	|| file_progress_show_target (ctx, d) == FILE_ABORT)
@@ -1084,7 +1081,9 @@ move_file_file (FileOpContext *ctx, const char *s, const char *d,
 	}
 
 	if (mc_rename (s, d) == 0) {
-	    return FILE_CONT;
+	    return progress_update_one (ctx, progress_count,
+	    				progress_bytes,
+	    				src_stats.st_size, 1);
 	}
     }
 #if 0
@@ -1111,6 +1110,8 @@ move_file_file (FileOpContext *ctx, const char *s, const char *d,
     if (return_status != FILE_CONT)
 	return return_status;
 
+    copy_done = TRUE;
+
     if ((return_status =
 	 file_progress_show_source (ctx, NULL)) != FILE_CONT
 	|| (return_status = file_progress_show (ctx, 0, 0)) != FILE_CONT)
@@ -1127,11 +1128,12 @@ move_file_file (FileOpContext *ctx, const char *s, const char *d,
 	return return_status;
     }
 
-    if (return_status == FILE_CONT)
+    if (!copy_done) {
 	return_status = progress_update_one (ctx,
 					     progress_count,
 					     progress_bytes,
 					     src_stats.st_size, 1);
+    }
 
     return return_status;
 }
@@ -1210,14 +1212,6 @@ move_dir_dir (FileOpContext *ctx, const char *s, const char *d,
 	return_status = FILE_CONT;
 	goto ret;
     }
-/* .ado: Drive, Do we need this anymore? */
-#ifdef WIN32
-    else {
-	/* EXDEV: cross device; does not work everywhere */
-	if (toupper (s[0]) != toupper (destdir[0]))
-	    goto w32try;
-    }
-#endif				/* WIN32 */
 
     if (errno != EXDEV) {
 	return_status =
@@ -1228,9 +1222,6 @@ move_dir_dir (FileOpContext *ctx, const char *s, const char *d,
 	    goto retry_rename;
 	goto ret;
     }
-#ifdef WIN32
-  w32try:
-#endif				/* WIN32 */
     /* Failed because of filesystem boundary -> copy dir instead */
     return_status =
 	copy_dir_dir (ctx, s, destdir, 0, 0, 1, 0, progress_count,
