@@ -1,23 +1,4 @@
-/* {{{ Copyright */
-
-/* File managing.  Important notes on this file:
- *  
- * About the use of dialogs in this file:
- *   If you want to add a new dialog box (or call a routine that pops
- *   up a dialog box), you have to provide a wrapper for background
- *   operations (ie, background operations have to up-call to the parent
- *   process).
- *
- *   For example, instead of using the message() routine, in this
- *   file, you should use one of the stubs that call message with the
- *   proper number of arguments (ie, message_1s, message_2s and so on).
- *
- *   Actually, that is a rule that should be followed by any routines
- *   that may be called from this module.
- *
- */
-
-/* File managing GUI for the text mode edition
+/* File management GUI for the text mode edition
  *
  * Copyright (C) 1994, 1995, 1996 The Free Software Foundation
  *  
@@ -49,45 +30,47 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ */
+
+/*
+ * Please note that all dialogs used here must be safe for background
+ * operations.
  */
 
-/* }}} */
 
 /* {{{ Include files */
 
 #include <config.h>
-/* Hack: the vfs code should not rely on this */
-#define WITH_FULL_PATHS 1
-
-#include <sys/types.h>
-#include <stdio.h>
 
 #include <errno.h>
 #include <ctype.h>
+#include <stdio.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
+
+#include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
 #include "global.h"
 #include "setup.h"		/* verbose */
-#include "dialog.h"		/* do_refresh */
+#include "dialog.h"		/* do_refresh() */
 #include "color.h"		/* dialog_colors */
-#include "dlg.h"		/* B_ENTER */
-#include "background.h"		/* message_3s */
 #include "widget.h"		/* WLabel */
 #define WANT_WIDGETS
 #include "main.h"		/* the_hint */
 #include "wtools.h"		/* QuickDialog */
-#include "panel.h"		/* cpanel */
+#include "panel.h"		/* current_panel */
 #include "fileopctx.h"		/* FILE_CONT */
 #include "filegui.h"
 #include "key.h"		/* get_event */
+#include "util.h"               /* strip_password() */
 
 /* }}} */
 
+/* Hack: the vfs code should not rely on this */
+#define WITH_FULL_PATHS 1
+
 /* This structure describes the UI and internal data required by a file
  * operation context.
  */
@@ -113,7 +96,7 @@ typedef struct {
     /* Query replace dialog */
 
     Dlg_head *replace_dlg;
-    char *replace_filename;
+    const char *replace_filename;
     int replace_result;
     struct stat *s_stat, *d_stat;
 } FileOpContextUI;
@@ -142,7 +125,7 @@ enum {
     REPLACE_ABORT,
     REPLACE_SIZE,
     REPLACE_REGET
-} FileReplaceCode;
+};
 
 static FileProgressStatus
 check_progress_buttons (FileOpContext *ctx)
@@ -182,15 +165,14 @@ check_progress_buttons (FileOpContext *ctx)
 /* {{{ File progress display routines */
 
 void
-file_op_context_create_ui (FileOpContext *ctx, FileOperation op,
-			   int with_eta)
+file_op_context_create_ui (FileOpContext *ctx, int with_eta)
 {
     FileOpContextUI *ui;
     int x_size;
     int minus;
     int eta_offset;
-    char *sixty;
-    char *fifteen;
+    const char *sixty;
+    const char *fifteen;
 
     g_return_if_fail (ctx != NULL);
     g_return_if_fail (ctx->ui == NULL);
@@ -212,8 +194,10 @@ file_op_context_create_ui (FileOpContext *ctx, FileOperation op,
     ui->eta_extra = with_eta ? WX_ETA_EXTRA : 0;
     x_size = (WX + 4) + ui->eta_extra;
 
-    ui->op_dlg = create_dlg (0, 0, WY - minus + 4, x_size, dialog_colors,
-			     NULL, NULL, op_names[op], DLG_CENTER);
+    ui->op_dlg =
+	create_dlg (0, 0, WY - minus + 4, x_size, dialog_colors, NULL,
+		    NULL, op_names[ctx->operation],
+		    DLG_CENTER | DLG_REVERSE);
 
     last_hint_line = the_hint->widget.y;
     if ((ui->op_dlg->y + ui->op_dlg->lines) > last_hint_line)
@@ -221,40 +205,37 @@ file_op_context_create_ui (FileOpContext *ctx, FileOperation op,
 
     add_widget (ui->op_dlg,
 		button_new (BY - minus, WX - 19 + eta_offset, FILE_ABORT,
-			    NORMAL_BUTTON, _("&Abort"), 0, 0, "abort"));
+			    NORMAL_BUTTON, _("&Abort"), 0));
     add_widget (ui->op_dlg,
 		button_new (BY - minus, 14 + eta_offset, FILE_SKIP,
-			    NORMAL_BUTTON, _("&Skip"), 0, 0, "skip"));
+			    NORMAL_BUTTON, _("&Skip"), 0));
 
     add_widget (ui->op_dlg, ui->progress_gauge[2] =
-		gauge_new (7, FCOPY_GAUGE_X, 0, 100, 0, "g-1"));
+		gauge_new (7, FCOPY_GAUGE_X, 0, 100, 0));
     add_widget (ui->op_dlg, ui->progress_label[2] =
-		label_new (7, FCOPY_LABEL_X, fifteen, "l-1"));
-    add_widget (ui->op_dlg, ui->bps_label =
-		label_new (7, WX, "", "bps-label"));
+		label_new (7, FCOPY_LABEL_X, fifteen));
+    add_widget (ui->op_dlg, ui->bps_label = label_new (7, WX, ""));
 
     add_widget (ui->op_dlg, ui->progress_gauge[1] =
-		gauge_new (8, FCOPY_GAUGE_X, 0, 100, 0, "g-2"));
+		gauge_new (8, FCOPY_GAUGE_X, 0, 100, 0));
     add_widget (ui->op_dlg, ui->progress_label[1] =
-		label_new (8, FCOPY_LABEL_X, fifteen, "l-2"));
-    add_widget (ui->op_dlg, ui->stalled_label =
-		label_new (8, WX, "", "stalled"));
+		label_new (8, FCOPY_LABEL_X, fifteen));
+    add_widget (ui->op_dlg, ui->stalled_label = label_new (8, WX, ""));
 
     add_widget (ui->op_dlg, ui->progress_gauge[0] =
-		gauge_new (6, FCOPY_GAUGE_X, 0, 100, 0, "g-3"));
+		gauge_new (6, FCOPY_GAUGE_X, 0, 100, 0));
     add_widget (ui->op_dlg, ui->progress_label[0] =
-		label_new (6, FCOPY_LABEL_X, fifteen, "l-3"));
-    add_widget (ui->op_dlg, ui->eta_label =
-		label_new (6, WX, "", "eta_label"));
+		label_new (6, FCOPY_LABEL_X, fifteen));
+    add_widget (ui->op_dlg, ui->eta_label = label_new (6, WX, ""));
 
     add_widget (ui->op_dlg, ui->file_string[1] =
-		label_new (4, FCOPY_GAUGE_X, sixty, "fs-l-1"));
+		label_new (4, FCOPY_GAUGE_X, sixty));
     add_widget (ui->op_dlg, ui->file_label[1] =
-		label_new (4, FCOPY_LABEL_X, fifteen, "fs-l-2"));
+		label_new (4, FCOPY_LABEL_X, fifteen));
     add_widget (ui->op_dlg, ui->file_string[0] =
-		label_new (3, FCOPY_GAUGE_X, sixty, "fs-x-1"));
+		label_new (3, FCOPY_GAUGE_X, sixty));
     add_widget (ui->op_dlg, ui->file_label[0] =
-		label_new (3, FCOPY_LABEL_X, fifteen, "fs-x-2"));
+		label_new (3, FCOPY_LABEL_X, fifteen));
 
     /* We will manage the dialog without any help, that's why
        we have to call init_dlg */
@@ -442,10 +423,11 @@ file_progress_show_bytes (FileOpContext *ctx, double done, double total)
 
 /* }}} */
 
-#define truncFileString(ui, s) name_trunc (s, ui->eta_extra + 47)
+#define truncFileString(ui, s)       name_trunc (s, ui->eta_extra + 47)
+#define truncFileStringSecure(ui, s) path_trunc (s, ui->eta_extra + 47)
 
 FileProgressStatus
-file_progress_show_source (FileOpContext *ctx, char *s)
+file_progress_show_source (FileOpContext *ctx, const char *s)
 {
     FileOpContextUI *ui;
 
@@ -458,10 +440,10 @@ file_progress_show_source (FileOpContext *ctx, char *s)
 
     if (s != NULL) {
 #ifdef WITH_FULL_PATHS
-	int i = strlen (cpanel->cwd);
+	int i = strlen (current_panel->cwd);
 
 	/* We remove the full path we have added before */
-	if (!strncmp (s, cpanel->cwd, i)) {
+	if (!strncmp (s, current_panel->cwd, i)) {
 	    if (s[i] == PATH_SEP)
 		s += i + 1;
 	}
@@ -478,7 +460,7 @@ file_progress_show_source (FileOpContext *ctx, char *s)
 }
 
 FileProgressStatus
-file_progress_show_target (FileOpContext *ctx, char *s)
+file_progress_show_target (FileOpContext *ctx, const char *s)
 {
     FileOpContextUI *ui;
 
@@ -491,7 +473,7 @@ file_progress_show_target (FileOpContext *ctx, char *s)
 
     if (s != NULL) {
 	label_set_text (ui->file_label[1], _("Target"));
-	label_set_text (ui->file_string[1], truncFileString (ui, s));
+	label_set_text (ui->file_string[1], truncFileStringSecure (ui, s));
 	return check_progress_buttons (ctx);
     } else {
 	label_set_text (ui->file_label[1], "");
@@ -501,7 +483,7 @@ file_progress_show_target (FileOpContext *ctx, char *s)
 }
 
 FileProgressStatus
-file_progress_show_deleting (FileOpContext *ctx, char *s)
+file_progress_show_deleting (FileOpContext *ctx, const char *s)
 {
     FileOpContextUI *ui;
 
@@ -513,7 +495,7 @@ file_progress_show_deleting (FileOpContext *ctx, char *s)
     ui = ctx->ui;
 
     label_set_text (ui->file_label[0], _("Deleting"));
-    label_set_text (ui->file_label[0], truncFileString (ui, s));
+    label_set_text (ui->file_label[0], truncFileStringSecure (ui, s));
     return check_progress_buttons (ctx);
 }
 
@@ -525,44 +507,48 @@ file_progress_show_deleting (FileOpContext *ctx, char *s)
  *   alex
  */
 static struct {
-    char *text;
+    const char *text;
     int ypos, xpos;
     int value;			/* 0 for labels */
-    char *tkname;
 } rd_widgets[] = {
     {
-    N_("Target file \"%s\" already exists!"), 3, 4, 0, "target-e"}, {
-    N_("&Abort"), BY + 3, 25, REPLACE_ABORT, "abort"}, {
-    N_("If &size differs"), BY + 1, 28, REPLACE_SIZE, "if-size"}, {
-    N_("Non&e"), BY, 47, REPLACE_NEVER, "none"}, {
-    N_("&Update"), BY, 36, REPLACE_UPDATE, "update"}, {
-    N_("A&ll"), BY, 28, REPLACE_ALWAYS, "all"}, {
-    N_("Overwrite all targets?"), BY, 4, 0, "over-label"}, {
-    N_("&Reget"), BY - 1, 28, REPLACE_REGET, "reget"}, {
-    N_("A&ppend"), BY - 2, 45, REPLACE_APPEND, "append"}, {
-    N_("&No"), BY - 2, 37, REPLACE_NO, "no"}, {
-    N_("&Yes"), BY - 2, 28, REPLACE_YES, "yes"}, {
-    N_("Overwrite this target?"), BY - 2, 4, 0, "overlab"}, {
-    N_("Target date: %s, size %d"), 6, 4, 0, "target-date"}, {
-    N_("Source date: %s, size %d"), 5, 4, 0, "source-date"}
+    N_("Target file \"%s\" already exists!"), 3, 4, 0}, {
+    N_("&Abort"), BY + 3, 25, REPLACE_ABORT}, {
+    N_("If &size differs"), BY + 1, 28, REPLACE_SIZE}, {
+    N_("Non&e"), BY, 47, REPLACE_NEVER}, {
+    N_("&Update"), BY, 36, REPLACE_UPDATE}, {
+    N_("A&ll"), BY, 28, REPLACE_ALWAYS}, {
+    N_("Overwrite all targets?"), BY, 4, 0}, {
+    N_("&Reget"), BY - 1, 28, REPLACE_REGET}, {
+    N_("A&ppend"), BY - 2, 45, REPLACE_APPEND}, {
+    N_("&No"), BY - 2, 37, REPLACE_NO}, {
+    N_("&Yes"), BY - 2, 28, REPLACE_YES}, {
+    N_("Overwrite this target?"), BY - 2, 4, 0}, {
+#if defined(_FILE_OFFSET_BITS) && _FILE_OFFSET_BITS == 64
+    N_("Target date: %s, size %llu"), 6, 4, 0}, {
+    N_("Source date: %s, size %llu"), 5, 4, 0}
+#else
+    N_("Target date: %s, size %u"), 6, 4, 0}, {
+    N_("Source date: %s, size %u"), 5, 4, 0}
+#endif
 };
 
 #define ADD_RD_BUTTON(i)\
 	add_widget (ui->replace_dlg,\
 		button_new (rd_widgets [i].ypos, rd_widgets [i].xpos, rd_widgets [i].value,\
-		NORMAL_BUTTON, rd_widgets [i].text, 0, 0, rd_widgets [i].tkname))
+		NORMAL_BUTTON, rd_widgets [i].text, 0))
 
 #define ADD_RD_LABEL(ui,i,p1,p2)\
 	g_snprintf (buffer, sizeof (buffer), rd_widgets [i].text, p1, p2);\
 	add_widget (ui->replace_dlg,\
-		label_new (rd_widgets [i].ypos, rd_widgets [i].xpos, buffer, rd_widgets [i].tkname))
+		label_new (rd_widgets [i].ypos, rd_widgets [i].xpos, buffer))
 
 static void
 init_replace (FileOpContext *ctx, enum OperationMode mode)
 {
     FileOpContextUI *ui;
     char buffer[BUF_SMALL];
-    char *title;
+    const char *title;
     static int rd_xlen = 60, rd_trunc = X_TRUNC;
 
 #ifdef ENABLE_NLS
@@ -626,7 +612,7 @@ init_replace (FileOpContext *ctx, enum OperationMode mode)
     /* FIXME - missing help node */
     ui->replace_dlg =
 	create_dlg (0, 0, 16, rd_xlen, alarm_colors, NULL, "[Replace]",
-		    title, DLG_CENTER);
+		    title, DLG_CENTER | DLG_REVERSE);
 
 
     ADD_RD_LABEL (ui, 0,
@@ -642,24 +628,24 @@ init_replace (FileOpContext *ctx, enum OperationMode mode)
 
     /* "this target..." widgets */
     if (!S_ISDIR (ui->d_stat->st_mode)) {
-	if ((ui->d_stat->st_size
-	     && ui->s_stat->st_size > ui->d_stat->st_size))
-	    ADD_RD_BUTTON (7);
+	if ((ctx->operation == OP_COPY) && (ui->d_stat->st_size != 0)
+	    && (ui->s_stat->st_size > ui->d_stat->st_size))
+	    ADD_RD_BUTTON (7);	/* reget */
 
-	ADD_RD_BUTTON (8);
+	ADD_RD_BUTTON (8);	/* Overwrite all targets? */
     }
     ADD_RD_BUTTON (9);
     ADD_RD_BUTTON (10);
     ADD_RD_LABEL (ui, 11, 0, 0);
 
     ADD_RD_LABEL (ui, 12, file_date (ui->d_stat->st_mtime),
-		  (int) ui->d_stat->st_size);
+		  (off_t) ui->d_stat->st_size);
     ADD_RD_LABEL (ui, 13, file_date (ui->s_stat->st_mtime),
-		  (int) ui->s_stat->st_size);
+		  (off_t) ui->s_stat->st_size);
 }
 
 void
-file_progress_set_stalled_label (FileOpContext *ctx, char *stalled_msg)
+file_progress_set_stalled_label (FileOpContext *ctx, const char *stalled_msg)
 {
     FileOpContextUI *ui;
 
@@ -675,7 +661,7 @@ file_progress_set_stalled_label (FileOpContext *ctx, char *stalled_msg)
 
 FileProgressStatus
 file_progress_real_query_replace (FileOpContext *ctx,
-				  enum OperationMode mode, char *destname,
+				  enum OperationMode mode, const char *destname,
 				  struct stat *_s_stat,
 				  struct stat *_d_stat)
 {
@@ -714,7 +700,7 @@ file_progress_real_query_replace (FileOpContext *ctx,
 	    return FILE_CONT;
 
     case REPLACE_REGET:
-	/* Carefull: we fall through and set do_append */
+	/* Careful: we fall through and set do_append */
 	ctx->do_reget = _d_stat->st_size;
 
     case REPLACE_APPEND:
@@ -744,12 +730,12 @@ static QuickWidget fmd_widgets[] = {
 #define	FMCB11 1
     /* follow symlinks and preserve Attributes must be the first */
     {quick_checkbox, 3, 64, 8, FMDY, N_("preserve &Attributes"), 9, 0,
-     0 /* &op_preserve */ , 0, "preserve"},
+     0 /* &op_preserve */ , 0, NULL},
     {quick_checkbox, 3, 64, 7, FMDY, N_("follow &Links"), 7, 0,
-     0 /* &file_mask_op_follow_links */ , 0, "follow"},
-    {quick_label, 3, 64, 5, FMDY, N_("to:"), 0, 0, 0, 0, "to"},
+     0 /* &file_mask_op_follow_links */ , 0, NULL},
+    {quick_label, 3, 64, 5, FMDY, N_("to:"), 0, 0, 0, 0, NULL},
     {quick_checkbox, 37, 64, 4, FMDY, N_("&Using shell patterns"), 0, 0,
-     0 /* &source_easy_patterns */ , 0, "using-shell"},
+     0 /* &source_easy_patterns */ , 0, NULL},
     {quick_input, 3, 64, 3, FMDY, "", 58,
      0, 0, 0, "input-def"},
 #define FMDI1 4
@@ -758,10 +744,10 @@ static QuickWidget fmd_widgets[] = {
     {quick_input, 3, 64, 6, FMDY, "", 58, 0,
      0, 0, "input2"},
 #define FMDI0 6
-    {quick_label, 3, 64, 2, FMDY, "", 0, 0, 0, 0, "ql"},
+    {quick_label, 3, 64, 2, FMDY, "", 0, 0, 0, 0, NULL},
 #define	FMBRGT 7
     {quick_button, 42, 64, 9, FMDY, N_("&Cancel"), 0, B_CANCEL, 0, 0,
-     "cancel"},
+     NULL},
 #undef SKIP
 #ifdef WITH_BACKGROUND
 # define SKIP 5
@@ -770,7 +756,7 @@ static QuickWidget fmd_widgets[] = {
 # define FMBLFT 9
 # define FMBMID 8
     {quick_button, 25, 64, 9, FMDY, N_("&Background"), 0, B_USER, 0, 0,
-     "back"},
+     NULL},
 #else				/* WITH_BACKGROUND */
 # define SKIP 4
 # define FMCB21 10
@@ -778,13 +764,13 @@ static QuickWidget fmd_widgets[] = {
 # define FMBLFT 8
 # undef  FMBMID
 #endif
-    {quick_button, 14, 64, 9, FMDY, N_("&OK"), 0, B_ENTER, 0, 0, "ok"},
+    {quick_button, 14, 64, 9, FMDY, N_("&OK"), 0, B_ENTER, 0, 0, NULL},
     {quick_checkbox, 42, 64, 8, FMDY, N_("&Stable Symlinks"), 0, 0,
-     0 /* &file_mask_stable_symlinks */ , 0, "stab-sym"},
+     0 /* &file_mask_stable_symlinks */ , 0, NULL},
     {quick_checkbox, 31, 64, 7, FMDY, N_("&Dive into subdir if exists"), 0,
      0,
-     0 /* &dive_into_subdirs */ , 0, "dive"},
-    {0}
+     0 /* &dive_into_subdirs */ , 0, NULL},
+    NULL_QuickWidget
 };
 
 static int
@@ -868,19 +854,20 @@ fmd_init_i18n (int force)
 }
 
 char *
-file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text,
-		  char *def_text, int only_one, int *do_background)
+file_mask_dialog (FileOpContext *ctx, FileOperation operation, const char *text,
+		  const char *def_text, int only_one, int *do_background)
 {
     int source_easy_patterns = easy_patterns;
-    char *source_mask, *orig_mask, *dest_dir;
+    char *source_mask, *orig_mask, *dest_dir, *tmpdest;
     const char *error;
+    char *def_text_secure;
     struct stat buf;
     int val;
     QuickDialog Quick_input;
 
     g_return_val_if_fail (ctx != NULL, NULL);
 #if 0
-    message_3s (1, __FUNCTION__, "text = `%s' \n def_text = `%s'", text,
+    message (1, __FUNCTION__, "text = `%s' \n def_text = `%s'", text,
 		def_text);
 #endif
     fmd_init_i18n (FALSE);
@@ -892,6 +879,9 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text,
     fmd_widgets[FMCB22].result = &ctx->stable_symlinks;
     fmd_widgets[FMCB21].result = &ctx->dive_into_subdirs;
 
+    /* filter out a possible password from def_text */
+    def_text_secure = strip_password (g_strdup (def_text), 1);
+
     /* Create the dialog */
 
     ctx->stable_symlinks = 0;
@@ -905,20 +895,23 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text,
     Quick_input.i18n = 1;
     Quick_input.widgets = fmd_widgets;
     fmd_widgets[FMDI0].text = text;
-    fmd_widgets[FMDI2].text = def_text;
+    fmd_widgets[FMDI2].text = def_text_secure;
     fmd_widgets[FMDI2].str_result = &dest_dir;
     fmd_widgets[FMDI1].str_result = &source_mask;
 
     *do_background = 0;
   ask_file_mask:
 
-    if ((val = quick_dialog_skip (&Quick_input, SKIP)) == B_CANCEL)
+    if ((val = quick_dialog_skip (&Quick_input, SKIP)) == B_CANCEL) {
+	g_free (def_text_secure);
 	return 0;
+    }
+    g_free (def_text_secure);
 
     if (ctx->follow_links)
-	ctx->stat_func = (mc_stat_fn) mc_stat;
+	ctx->stat_func = mc_stat;
     else
-	ctx->stat_func = (mc_stat_fn) mc_lstat;
+	ctx->stat_func = mc_lstat;
 
     if (ctx->op_preserve) {
 	ctx->preserve = 1;
@@ -934,8 +927,7 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text,
 
     orig_mask = source_mask;
     if (!dest_dir || !*dest_dir) {
-	if (source_mask)
-	    g_free (source_mask);
+	g_free (source_mask);
 	return dest_dir;
     }
     if (source_easy_patterns) {
@@ -953,14 +945,17 @@ file_mask_dialog (FileOpContext *ctx, FileOperation operation, char *text,
 				&ctx->rx);
 
     if (error) {
-	message_3s (1, MSG_ERROR, _("Invalid source pattern `%s' \n %s "),
+	message (1, MSG_ERROR, _("Invalid source pattern `%s' \n %s "),
 		    orig_mask, error);
-	if (orig_mask)
-	    g_free (orig_mask);
+	g_free (orig_mask);
 	goto ask_file_mask;
     }
-    if (orig_mask)
-	g_free (orig_mask);
+    g_free (orig_mask);
+
+    tmpdest = dest_dir;
+    dest_dir = tilde_expand(tmpdest);
+    g_free(tmpdest);
+
     ctx->dest_mask = strrchr (dest_dir, PATH_SEP);
     if (ctx->dest_mask == NULL)
 	ctx->dest_mask = dest_dir;
