@@ -16,41 +16,38 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #include <config.h>
+
+#include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
-#include <sys/param.h>		/* Required by tree.h */
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(HAS_NO_TERMIOS_H)
-#    include <termios.h>
-#endif
      /*
       * If TIOCGWINSZ supported, make it available here, because window-
       * resizing code depends on it...
       */
-#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(HAS_NO_SYS_IOCTL_H)
-#	include <sys/ioctl.h>
+#ifdef HAVE_SYS_IOCTL_H
+#   include <sys/ioctl.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
 #endif
-#include <signal.h>
-#include "tty.h"
+#include <unistd.h>
+
 #include "global.h"
+#include "tty.h"		/* COLS */
 #include "win.h"
 #include "color.h"
 #include "key.h"
-
-#include "dlg.h"
+#include "dialog.h"
 #include "widget.h"
 #include "command.h"
-
-#include "dialog.h"		/* For do_refresh() */
 #include "profile.h"		/* For sync_profiles() */
 #include "mouse.h"
 #define WANT_WIDGETS
@@ -62,25 +59,16 @@
 /* Needed for the extern declarations of integer parameters */
 #include "dir.h"
 #include "panel.h"		/* The Panel widget */
-#include "file.h"
 #include "cons.saver.h"
 #include "layout.h"
 #include "info.h"		/* The Info widget */
 #include "view.h"		/* The view widget */
 
-#define WANT_WIDGETS
 #include "setup.h"		/* For save_setup() */
 
-#include "x.h"
-
-/* "$Id: layout.c,v 1.23 2001/04/06 21:48:12 proskin Exp $" */
-
 /* Controls the display of the rotating dash on the verbose mode */
 int nice_rotating_dash = 1;
 
-/* If set, then we have to call the layout_change routine from main */
-int layout_do_change = 0;
-
 /* Set if the panels are split horizontally */
 int horizontal_split = 0;
 
@@ -99,14 +87,14 @@ int output_lines = 0;
 /* Set if the command prompt is to be displayed */
 int command_prompt = 1;
 
-/* Set if the nice and usefull keybar is visible */
+/* Set if the nice and useful keybar is visible */
 int keybar_visible = 1;
 
 /* Set if the nice message (hint) bar is visible */
 int message_visible = 1;
 
-/* Set if you want the message bar shown in xterm title bar to save space */
-int xterm_hintbar = 0;
+/* Set to show current working dir in xterm window title */
+int xterm_title = 1;
 
 /* The starting line for the output of the subprogram */
 int output_start_y = 0;
@@ -116,7 +104,7 @@ int output_start_y = 0;
 /* Janne gets around this, we will only manage two of them :-) */
 #define MAX_VIEWS 2
 
-struct {
+static struct {
     int    type;
     Widget *widget;
 } panels [MAX_VIEWS];
@@ -136,7 +124,7 @@ static int _output_lines;
 static int _command_prompt;
 static int _keybar_visible;
 static int _message_visible;
-static int _xterm_hintbar;
+static int _xterm_title;
 static int _permission_mode;
 static int _filetype_mode;
 
@@ -149,45 +137,43 @@ static int height;
 #define BY      12
 
 #define B_2LEFT B_USER
-#define B_2RIGHT B_USER + 1
-#define B_PLUS B_USER + 2
-#define B_MINUS B_USER + 3
+#define B_2RIGHT (B_USER + 1)
+#define B_PLUS (B_USER + 2)
+#define B_MINUS (B_USER + 3)
 
 static Dlg_head *layout_dlg;
 
-static char *s_split_direction [2] = {
+static const char *s_split_direction [2] = {
     N_("&Vertical"), 
     N_("&Horizontal")
 };
-WRadio *radio_widget;
+
+static WRadio *radio_widget;
 
 static struct {
-    char   *text;
+    const char   *text;
     int    *variable;
     WCheck *widget;
-    char   *tkname;
 } check_options [] = {
-    { N_("&Xterm hintbar"),    &xterm_hintbar,   0, "h" },
-    { N_("h&Intbar visible"),  &message_visible, 0, "v" },
-    { N_("&Keybar visible"),   &keybar_visible,  0, "k" },
-    { N_("command &Prompt"),   &command_prompt,  0, "p" },
-    { N_("show &Mini status"), &show_mini_info,  0, "m" },
-    { N_("menu&Bar visible"),  &menubar_visible, 0, "me" },
-    { N_("&Equal split"),      &equal_split,     0, "eq" },
-    { N_("pe&Rmissions"),      &permission_mode, 0, "pr" },
-    { N_("&File types"),       &filetype_mode,   0, "ft" },
-    { 0, 0, 0, 0 }
+    { N_("&Xterm window title"), &xterm_title,   0 },
+    { N_("h&Intbar visible"),  &message_visible, 0 },
+    { N_("&Keybar visible"),   &keybar_visible,  0 },
+    { N_("command &Prompt"),   &command_prompt,  0 },
+    { N_("show &Mini status"), &show_mini_info,  0 },
+    { N_("menu&Bar visible"),  &menubar_visible, 0 },
+    { N_("&Equal split"),      &equal_split,     0 },
+    { N_("pe&Rmissions"),      &permission_mode, 0 },
+    { N_("&File types"),       &filetype_mode,   0 },
+    { 0, 0, 0 }
 };
 
 static int first_width, second_width;
-static char *layout_title, *title1, *title2, *title3, *output_lines_label;
+static const char *output_lines_label;
 
 static WButton *bleft_widget, *bright_widget;
 
 /* Declarations for static functions */
-#ifdef PORT_NEEDS_CHANGE_SCREEN_SIZE
 static void low_level_change_screen_size (void);
-#endif /* PORT_NEEDS_CHANGE_SCREEN_SIZE */
 
 static void _check_split (void)
 {
@@ -225,80 +211,76 @@ static void update_split (void)
    
     attrset (COLOR_NORMAL);
     dlg_move (layout_dlg, 6, 6);
-    printw ("%03d", _first_panel_size);
+    tty_printf ("%03d", _first_panel_size);
     dlg_move (layout_dlg, 6, 18);
     if (_horizontal_split)
-	printw ("%03d", height - _first_panel_size);
+	tty_printf ("%03d", height - _first_panel_size);
     else
-	printw ("%03d", COLS - _first_panel_size);
+	tty_printf ("%03d", COLS - _first_panel_size);
 }
 
-static int b2left_cback (int action, void *data)
+static int b2left_cback (int action)
 {
+    (void) action;
+
     if (_equal_split){
 	/* Turn equal split off */
 	_equal_split = 0;
 	check_options [6].widget->state = check_options [6].widget->state & ~C_BOOL;
-	dlg_select_widget (layout_dlg, check_options [6].widget);
-	dlg_select_widget (layout_dlg, bleft_widget);
+	dlg_select_widget (check_options [6].widget);
+	dlg_select_widget (bleft_widget);
     }
     _first_panel_size++;
     return 0;
 }
 
-static int b2right_cback (int action, void *data)
+static int b2right_cback (int action)
 {
+    (void) action;
+
     if (_equal_split){
 	/* Turn equal split off */
 	_equal_split = 0;
 	check_options [6].widget->state = check_options [6].widget->state & ~C_BOOL;
-	dlg_select_widget (layout_dlg, check_options [6].widget);
-	dlg_select_widget (layout_dlg, bright_widget);
+	dlg_select_widget (check_options [6].widget);
+	dlg_select_widget (bright_widget);
     }
     _first_panel_size--;
     return 0;
 }
 
-static int bplus_cback (int action, void *data)
+static int bplus_cback (int action)
 {
+    (void) action;
+
     if (_output_lines < 99)
 	_output_lines++;
     return 0;
 }
 
-static int bminus_cback (int action, void *data)
+static int bminus_cback (int action)
 {
+    (void) action;
+
     if (_output_lines > 0)
 	_output_lines--;
     return 0;
 }
 
-static int layout_callback (struct Dlg_head *h, int Id, int Msg)
+static cb_ret_t
+layout_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
 {
-    switch (Msg){
+    switch (msg) {
     case DLG_DRAW:
-#ifndef HAVE_X
     	/*When repainting the whole dialog (e.g. with C-l) we have to
     	  update everything*/
+	common_dialog_repaint (h);
+
    	old_first_panel_size = -1;
     	old_horizontal_split = -1;
     	old_output_lines     = -1;
-	attrset (COLOR_NORMAL);
-	dlg_erase (h);
-	draw_box (h, 1, 2, h->lines - 2, h->cols - 4);
-	draw_box (h, 2, 4, 6, first_width);
-	draw_box (h, 8, 4, 4, first_width);
-	draw_box (h, 2, 5 + first_width, 10, second_width);
 
 	attrset (COLOR_HOT_NORMAL);
-	dlg_move (h, 1, (h->cols - strlen(layout_title))/2);
-	addstr (layout_title);
-	dlg_move (h, 2, 5);
-	addstr (title1);
-	dlg_move (h, 8, 5);
-	addstr (title2);
-	dlg_move (h, 2, 6 + first_width);
-	addstr (title3);
 	update_split ();
 	dlg_move (h, 6, 13);
 	addch ('=');
@@ -309,30 +291,26 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 		dlg_move (h, 9, 16 + first_width);
 		addstr (output_lines_label);
 		dlg_move (h, 9, 10 + first_width);
-		printw ("%02d", _output_lines);
+		tty_printf ("%02d", _output_lines);
 	    }
 	}
-#endif
-	break;
+	return MSG_HANDLED;
 
     case DLG_POST_KEY:
 	_filetype_mode = check_options [8].widget->state & C_BOOL;
 	_permission_mode = check_options [7].widget->state & C_BOOL;
-#ifndef HAVE_X
 	_equal_split = check_options [6].widget->state & C_BOOL;
-#endif
 	_menubar_visible = check_options [5].widget->state & C_BOOL;
 	_command_prompt = check_options [4].widget->state & C_BOOL;
 	_keybar_visible = check_options [2].widget->state & C_BOOL;
 	_message_visible = check_options [1].widget->state & C_BOOL;
-	_xterm_hintbar = check_options [0].widget->state & C_BOOL;
+	_xterm_title = check_options [0].widget->state & C_BOOL;
 	if (console_flag){
 	    int minimum;
 	    if (_output_lines < 0)
 		_output_lines = 0;
 	    height = LINES - _keybar_visible - _command_prompt -
 		     _menubar_visible - _output_lines - _message_visible;
-	    if (_message_visible && _xterm_hintbar && xterm_flag) height++;
 	    minimum = MINHEIGHT * (1 + _horizontal_split);
 	    if (height < minimum){
 		_output_lines -= minimum - height;
@@ -341,7 +319,6 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 	} else {
 	    height = LINES - _keybar_visible - _command_prompt -
 		_menubar_visible - _output_lines - _message_visible;
-	    if (_message_visible && _xterm_hintbar && xterm_flag) height++;
 	}
 	if (_horizontal_split != radio_widget->sel){
 	    _horizontal_split = radio_widget->sel;
@@ -356,132 +333,133 @@ static int layout_callback (struct Dlg_head *h, int Id, int Msg)
 		old_output_lines = _output_lines;
 		attrset (COLOR_NORMAL);
 		dlg_move (h, 9, 10 + first_width);
-		printw ("%02d", _output_lines);
+		tty_printf ("%02d", _output_lines);
 	    }
 	}
-	break;
+	return MSG_HANDLED;
 
-    case DLG_END:
-	break;
+    default:
+	return default_dlg_callback (h, msg, parm);
     }
-    return 0;
 }
 
-static void init_layout (void)
+static void
+init_layout (void)
 {
     static int i18n_layt_flag = 0;
-	static int b1, b2, b3;
-    int i = sizeof (s_split_direction) / sizeof(char*) ;
-	char* ok_button = _("&Ok");
-	char* cancel_button = _("&Cancel");
-	char* save_button = _("&Save");
-    
-    if (!i18n_layt_flag)
-    {
-		register int l1;
-		
-		first_width = 19; /* length of line with '<' '>' buttons */
-		
-		layout_title = _(" Layout ");
-		title1 = _(" Panel split ");
-		title2 = _(" Highlight... ");
-		title3 = _(" Other options ");
-		output_lines_label = _("output lines");
-		
-		while (i--)
-		{
-		    s_split_direction [i] = _(s_split_direction [i]);
-			l1 = strlen (s_split_direction [i]) + 7;
-			if (l1 > first_width)
-				first_width = l1;
-		}
+    static int b1, b2, b3;
+    int i = sizeof (s_split_direction) / sizeof (char *);
+    const char *ok_button = _("&OK");
+    const char *cancel_button = _("&Cancel");
+    const char *save_button = _("&Save");
+    static const char *title1, *title2, *title3;
 
-		for (i = 0; i <= 8; i++)
-		{
-			check_options[i].text = _(check_options[i].text);
-			l1 = strlen (check_options[i].text) + 7;
-			if (l1 > first_width)
-				first_width = l1;
-		}
+    if (!i18n_layt_flag) {
+	register int l1;
 
-		l1 = strlen (title1) + 1;
-		if (l1 > first_width)
-			first_width = l1;
-		
-		l1 = strlen (title2) + 1;
-		if (l1 > first_width)
-			first_width = l1;
-		
+	first_width = 19;	/* length of line with '<' '>' buttons */
 
-		second_width = strlen (title3) + 1;
-		for (i = 0; i < 6; i++)
-		{
-			check_options[i].text = _(check_options[i].text);
-			l1 = strlen (check_options[i].text) + 7;
-			if (l1 > second_width)
-				second_width = l1;
-		}
-		if (console_flag)
-		{
-			l1 = strlen (output_lines_label) + 13;
-			if (l1 > second_width)
-				second_width = l1;
-		}
+	title1 = _(" Panel split ");
+	title2 = _(" Highlight... ");
+	title3 = _(" Other options ");
+	output_lines_label = _("output lines");
 
-		/* 
-		 * alex@bcs.zp.ua:
-		 * To be completely correct, one need to check if layout_title
-		 * does not exceed dialog length and total length of 3 buttons
-		 * allows their placement in one row. But assuming this dialog
-		 * is wide enough, I don't include such a tests.
-		 *
-		 * Now the last thing to do - properly space buttons...
-		 */
-		l1 = 11 + strlen (ok_button)   /* 14 - all brackets and inner space */
-		 	+ strlen (save_button)     /* notice: it is 3 char less because */
-			+ strlen (cancel_button);  /* of '&' char in button text */
-		
-		i = (first_width + second_width - l1) / 4;
-		b1 = 5 + i;
-		b2 = b1 + strlen(ok_button) + i + 6;
-		b3 = b2 + strlen(save_button) + i + 4;
+	while (i--) {
+	    s_split_direction[i] = _(s_split_direction[i]);
+	    l1 = strlen (s_split_direction[i]) + 7;
+	    if (l1 > first_width)
+		first_width = l1;
+	}
 
-		i18n_layt_flag = 1;
+	for (i = 0; i <= 8; i++) {
+	    check_options[i].text = _(check_options[i].text);
+	    l1 = strlen (check_options[i].text) + 7;
+	    if (l1 > first_width)
+		first_width = l1;
+	}
+
+	l1 = strlen (title1) + 1;
+	if (l1 > first_width)
+	    first_width = l1;
+
+	l1 = strlen (title2) + 1;
+	if (l1 > first_width)
+	    first_width = l1;
+
+
+	second_width = strlen (title3) + 1;
+	for (i = 0; i < 6; i++) {
+	    check_options[i].text = _(check_options[i].text);
+	    l1 = strlen (check_options[i].text) + 7;
+	    if (l1 > second_width)
+		second_width = l1;
+	}
+	if (console_flag) {
+	    l1 = strlen (output_lines_label) + 13;
+	    if (l1 > second_width)
+		second_width = l1;
+	}
+
+	/* 
+	 * alex@bcs.zp.ua:
+	 * To be completely correct, one need to check if the title
+	 * does not exceed dialog length and total length of 3 buttons
+	 * allows their placement in one row. But assuming this dialog
+	 * is wide enough, I don't include such a tests.
+	 *
+	 * Now the last thing to do - properly space buttons...
+	 */
+	l1 = 11 + strlen (ok_button)	/* 14 - all brackets and inner space */
+	    +strlen (save_button)	/* notice: it is 3 char less because */
+	    +strlen (cancel_button);	/* of '&' char in button text */
+
+	i = (first_width + second_width - l1) / 4;
+	b1 = 5 + i;
+	b2 = b1 + strlen (ok_button) + i + 6;
+	b3 = b2 + strlen (save_button) + i + 4;
+
+	i18n_layt_flag = 1;
     }
 
-    layout_dlg = create_dlg (0, 0, 15, first_width + second_width + 9, 
-		dialog_colors, layout_callback,
-		"[Layout]", "layout", DLG_CENTER | DLG_GRID);
-			     
-    x_set_dialog_title (layout_dlg, _("Layout"));
+    layout_dlg =
+	create_dlg (0, 0, 15, first_width + second_width + 9,
+		    dialog_colors, layout_callback, "[Layout]",
+		    _("Layout"), DLG_CENTER | DLG_REVERSE);
+
+    add_widget (layout_dlg, groupbox_new (4, 2, first_width, 6, title1));
+    add_widget (layout_dlg, groupbox_new (4, 8, first_width, 4, title2));
+    add_widget (layout_dlg,
+		groupbox_new (5 + first_width, 2, second_width, 10,
+			      title3));
 
     add_widget (layout_dlg,
-		button_new (BY, b3, B_CANCEL, NORMAL_BUTTON, cancel_button, 0, 0, "c"));
+		button_new (BY, b3, B_CANCEL, NORMAL_BUTTON, cancel_button,
+			    0));
     add_widget (layout_dlg,
-		button_new (BY, b2, B_EXIT, NORMAL_BUTTON, save_button, 0, 0, "s"));
+		button_new (BY, b2, B_EXIT, NORMAL_BUTTON, save_button,
+			    0));
     add_widget (layout_dlg,
-		button_new (BY, b1, B_ENTER, DEFPUSH_BUTTON, ok_button, 0, 0, "o"));
-#ifndef HAVE_X
-    if (console_flag){
+		button_new (BY, b1, B_ENTER, DEFPUSH_BUTTON, ok_button,
+			    0));
+    if (console_flag) {
 	add_widget (layout_dlg,
-		    button_new (9, 12 + first_width, B_MINUS, NARROW_BUTTON, "&-",
-			bminus_cback, 0, NULL));
+		    button_new (9, 12 + first_width, B_MINUS,
+				NARROW_BUTTON, "&-", bminus_cback));
 	add_widget (layout_dlg,
-		    button_new (9, 7 + first_width, B_PLUS, NARROW_BUTTON, "&+", 
-			bplus_cback, 0, NULL));
+		    button_new (9, 7 + first_width, B_PLUS, NARROW_BUTTON,
+				"&+", bplus_cback));
     }
-#endif    
+#define XTRACT(i) *check_options[i].variable, check_options[i].text
 
-#define XTRACT(i) *check_options[i].variable, check_options[i].text, check_options[i].tkname
-
-    for (i = 0; i < 6; i++){
-	check_options [i].widget = check_new (8 - i, 7 + first_width, XTRACT(i));
-	add_widget (layout_dlg, check_options [i].widget);
+    for (i = 0; i < 6; i++) {
+	check_options[i].widget =
+	    check_new (8 - i, 7 + first_width, XTRACT (i));
+	add_widget (layout_dlg, check_options[i].widget);
     }
-    check_options [8].widget = check_new (10, 6, XTRACT(8));
-    add_widget (layout_dlg, check_options [8].widget);
-    check_options [7].widget = check_new (9, 6, XTRACT(7));
-    add_widget (layout_dlg, check_options [7].widget);
+    check_options[8].widget = check_new (10, 6, XTRACT (8));
+    add_widget (layout_dlg, check_options[8].widget);
+    check_options[7].widget = check_new (9, 6, XTRACT (7));
+    add_widget (layout_dlg, check_options[7].widget);
 
     _filetype_mode = filetype_mode;
     _permission_mode = permission_mode;
@@ -490,45 +468,42 @@ static void init_layout (void)
     _command_prompt = command_prompt;
     _keybar_visible = keybar_visible;
     _message_visible = message_visible;
-    _xterm_hintbar = xterm_hintbar;
-#ifndef HAVE_X
-    bright_widget = button_new(6, 15, B_2RIGHT, NARROW_BUTTON, "&>", b2right_cback, 0, ">");
+    _xterm_title = xterm_title;
+    bright_widget =
+	button_new (6, 15, B_2RIGHT, NARROW_BUTTON, "&>", b2right_cback);
     add_widget (layout_dlg, bright_widget);
-    bleft_widget = button_new (6, 9, B_2LEFT, NARROW_BUTTON, "&<", b2left_cback, 0, "<");
+    bleft_widget =
+	button_new (6, 9, B_2LEFT, NARROW_BUTTON, "&<", b2left_cback);
     add_widget (layout_dlg, bleft_widget);
-    check_options [6].widget = check_new (5, 6, XTRACT(6));
-#endif
+    check_options[6].widget = check_new (5, 6, XTRACT (6));
     old_first_panel_size = -1;
     old_horizontal_split = -1;
-    old_output_lines     = -1;
-    
+    old_output_lines = -1;
+
     _first_panel_size = first_panel_size;
     _output_lines = output_lines;
-#ifndef HAVE_X
-    add_widget (layout_dlg, check_options [6].widget);
-    radio_widget = radio_new (3, 6, 2, s_split_direction, 1, "r");
+    add_widget (layout_dlg, check_options[6].widget);
+    radio_widget = radio_new (3, 6, 2, s_split_direction);
     add_widget (layout_dlg, radio_widget);
     radio_widget->sel = horizontal_split;
-#endif
 }
 
-void layout_change (void)
+static void
+layout_change (void)
 {
     setup_panels ();
-    layout_do_change = 0;
-#ifndef HAVE_X
     /* re-init the menu, because perhaps there was a change in the way 
-	how the panel are split (horizontal/vertical). */
-    done_menu();
-    init_menu();
-    menubar_arrange(the_menubar);
-#endif
+       how the panel are split (horizontal/vertical). */
+    done_menu ();
+    init_menu ();
+    menubar_arrange (the_menubar);
 }
 
 void layout_cmd (void)
 {
     int result;
     int i;
+    int layout_do_change = 0;
 
     init_layout ();
     run_dlg (layout_dlg);
@@ -538,12 +513,10 @@ void layout_cmd (void)
 	for (i = 0; check_options [i].text; i++)
 	    if (check_options [i].widget)
 		*check_options [i].variable = check_options [i].widget->state & C_BOOL;
-#ifndef HAVE_X		
 	horizontal_split = radio_widget->sel;
 	first_panel_size = _first_panel_size;
 	output_lines = _output_lines;
 	layout_do_change = 1;
-#endif	
     }
     if (result == B_EXIT){
 	save_layout ();
@@ -574,17 +547,10 @@ static void check_split (void)
     }
 }
 
-int panel_event    (Gpm_Event *event, WPanel *panel);
-int menu_bar_event (Gpm_Event *event, void *);
-extern char *prompt;
-
-#ifndef HAVE_X
 #ifdef HAVE_SLANG
-void init_curses ()
+void
+init_curses ()
 {
-    extern int force_ugly_line_drawing;
-    extern int SLtt_Has_Alt_Charset;
-    SLtt_get_terminfo ();
 #ifndef HAS_ACS_AS_PCCHARS
     if (force_ugly_line_drawing)
 	SLtt_Has_Alt_Charset = 0;
@@ -596,36 +562,73 @@ void init_curses ()
     nodelay (stdscr, FALSE);
 }
 #else
+static const struct {
+    int acscode;
+    int character;
+} acs_approx [] = {
+    { 'q',  '-' }, /* ACS_HLINE */
+    { 'x',  '|' }, /* ACS_VLINE */
+    { 'l',  '+' }, /* ACS_ULCORNER */
+    { 'k',  '+' }, /* ACS_URCORNER */
+    { 'm',  '+' }, /* ACS_LLCORNER */
+    { 'j',  '+' }, /* ACS_LRCORNER */
+    { 'a',  '#' }, /* ACS_CKBOARD */
+    { 'u',  '+' }, /* ACS_RTEE */
+    { 't',  '+' }, /* ACS_LTEE */
+    { 'w',  '+' }, /* ACS_TTEE */
+    { 'v',  '+' }, /* ACS_BTEE */
+    { 'n',  '+' }, /* ACS_PLUS */
+    { 0, 0 } };
+
 void init_curses (void)
 {
+    int i;
     initscr();
-    if (!status_using_ncurses)
-	do_enter_ca_mode ();
+#ifdef HAVE_ESCDELAY
+    /*
+     * If ncurses exports the ESCDELAY variable, it should be set to
+     * a low value, or you'll experience a delay in processing escape
+     * sequences that are recognized by mc (e.g. Esc-Esc).  On the other
+     * hand, making ESCDELAY too small can result in some sequences
+     * (e.g. cursor arrows) being reported as separate keys under heavy
+     * processor load, and this can be a problem if mc hasn't learned
+     * them in the "Learn Keys" dialog.  The value is in milliseconds.
+     */
+    ESCDELAY = 200;
+#endif /* HAVE_ESCDELAY */
+    do_enter_ca_mode ();
     mc_raw_mode ();
     noecho ();
     keypad (stdscr, TRUE);
     nodelay (stdscr, FALSE);
     init_colors ();
+    if (force_ugly_line_drawing) {
+	for (i = 0; acs_approx[i].acscode != 0; i++) {
+	    acs_map[acs_approx[i].acscode] = acs_approx[i].character;
+	}
+    }
 }
 #endif /* ! HAVE_SLANG */
-void done_screen ()                                                      
-{                                                                               
-    if (!(quit & SUBSHELL_EXIT))                                                
-	clr_scr ();                                                             
-    reset_shell_mode ();                                                        
-    mc_noraw_mode ();                                                           
-    if (use_mouse_p)                                                            
-	shut_mouse ();                                                          
-    keypad (stdscr, FALSE);                                                     
-}                                                                               
-#else
-void init_curses ()
+
+void
+clr_scr (void)
 {
+    standend ();
+    dlg_erase (midnight_dlg);
+    mc_refresh ();
+    doupdate ();
 }
-void done_screen ()
+
+void
+done_screen ()
 {
+    if (!(quit & SUBSHELL_EXIT))
+	clr_scr ();
+    reset_shell_mode ();
+    mc_noraw_mode ();
+    keypad (stdscr, FALSE);
+    done_colors ();
 }
-#endif /* HAVE_X */
 
 static void
 panel_do_cols (int index)
@@ -634,134 +637,106 @@ panel_do_cols (int index)
 	set_panel_formats ((WPanel *) panels [index].widget);
     else {
 	panel_update_cols (panels [index].widget, frame_half);
-
-	/* In case of an Hex Quick View must review line length */
-	if (get_display_type (index) == view_quick)
-	    view_update_bytes_per_line ((WView *) panels [index].widget);
     }
 }
 
-#ifdef HAVE_X
 void
 setup_panels (void)
-{
-    Widget *w = panels [0].widget;
-
-    winput_set_origin (&cmdline->input, 0, 60);
-
-    /* Only needed by the startup code */
-    if (panels [0].type == view_listing){
-	x_panel_set_size (0);
-    }
-    
-    if (panels [1].type == view_listing){
-	x_panel_set_size (1);
-    }
-
-    load_hint ();
-}
-
-#else
-
-void setup_panels (void)
 {
     int start_y;
     int promptl;		/* the prompt len */
 
-    if (console_flag){
+    if (console_flag) {
 	int minimum;
 	if (output_lines < 0)
 	    output_lines = 0;
-	height = LINES - keybar_visible - command_prompt - menubar_visible
-	         - output_lines - message_visible;
-	if (message_visible && xterm_hintbar && xterm_flag) height++;
+	height =
+	    LINES - keybar_visible - command_prompt - menubar_visible -
+	    output_lines - message_visible;
 	minimum = MINHEIGHT * (1 + horizontal_split);
-	if (height < minimum){
+	if (height < minimum) {
 	    output_lines -= minimum - height;
 	    height = minimum;
 	}
     } else {
-	height = LINES - menubar_visible - command_prompt -
-	    keybar_visible - message_visible;
-	if (message_visible && xterm_hintbar && xterm_flag) height++;
+	height =
+	    LINES - menubar_visible - command_prompt - keybar_visible -
+	    message_visible;
     }
     check_split ();
     start_y = menubar_visible;
 
     /* The column computing is defered until panel_do_cols */
-    if (horizontal_split){
-	widget_set_size (panels [0].widget, start_y, 0, 
-			 first_panel_size, 0);
-			
-	widget_set_size (panels [1].widget, start_y+first_panel_size, 0,
-			 height-first_panel_size, 0);
+    if (horizontal_split) {
+	widget_set_size (panels[0].widget, start_y, 0, first_panel_size,
+			 0);
+
+	widget_set_size (panels[1].widget, start_y + first_panel_size, 0,
+			 height - first_panel_size, 0);
     } else {
 	int first_x = first_panel_size;
 
-	widget_set_size (panels [0].widget, start_y, 0,
-			 height, 0);
+	widget_set_size (panels[0].widget, start_y, 0, height, 0);
+
+	widget_set_size (panels[1].widget, start_y, first_x, height, 0);
 
-	widget_set_size (panels [1].widget, start_y, first_x,
-			 height, 0);
-			
     }
     panel_do_cols (0);
     panel_do_cols (1);
-    
+
     promptl = strlen (prompt);
 
     widget_set_size (&the_menubar->widget, 0, 0, 1, COLS);
 
     if (command_prompt) {
-	    widget_set_size (&cmdline->input.widget,
-			     LINES-1-keybar_visible, promptl,
-			     1, COLS-promptl-(keybar_visible ? 0 : 1));
-	    winput_set_origin (&cmdline->input, promptl, COLS-promptl-(keybar_visible ? 0 : 1));
-	    widget_set_size (&the_prompt->widget,
-			     LINES-1-keybar_visible, 0,
-			     1, promptl);
+	widget_set_size (&cmdline->widget, LINES - 1 - keybar_visible,
+			 promptl, 1,
+			 COLS - promptl - (keybar_visible ? 0 : 1));
+	winput_set_origin (cmdline, promptl,
+			   COLS - promptl - (keybar_visible ? 0 : 1));
+	widget_set_size (&the_prompt->widget, LINES - 1 - keybar_visible,
+			 0, 1, promptl);
     } else {
-	    widget_set_size (&cmdline->input.widget, 0, 0, 0, 0);
-	    winput_set_origin (&cmdline->input, 0, 0);
-	    widget_set_size (&the_prompt->widget, LINES, COLS, 0, 0);
-    }			     
+	widget_set_size (&cmdline->widget, 0, 0, 0, 0);
+	winput_set_origin (cmdline, 0, 0);
+	widget_set_size (&the_prompt->widget, LINES, COLS, 0, 0);
+    }
+
+    widget_set_size ((Widget *) the_bar, LINES - 1, 0, keybar_visible, COLS);
+    buttonbar_set_visible (the_bar, keybar_visible);
 
-    widget_set_size (&the_bar->widget, LINES-1, 0, 1, COLS);
-    the_bar->visible = keybar_visible;
-    
     /* Output window */
-    if (console_flag && output_lines){
-	output_start_y = LINES -command_prompt-keybar_visible-
-	    output_lines;
+    if (console_flag && output_lines) {
+	output_start_y =
+	    LINES - command_prompt - keybar_visible - output_lines;
 	show_console_contents (output_start_y,
-			       LINES-output_lines-keybar_visible-1,
-			       LINES-keybar_visible-1);
-    } 
-    if (message_visible && (!xterm_hintbar || !xterm_flag))
-	widget_set_size (&the_hint->widget, height+start_y, 0, 1,COLS);
-    else
+			       LINES - output_lines - keybar_visible - 1,
+			       LINES - keybar_visible - 1);
+    }
+    if (message_visible) {
+	widget_set_size (&the_hint->widget, height + start_y, 0, 1, COLS);
+	set_hintbar ("");	/* clean up the line */
+    } else
 	widget_set_size (&the_hint->widget, 0, 0, 0, 0);
-    
-    load_hint ();
+
+    load_hint (1);
+    update_xterm_title_path ();
 }
-#endif
 
 void flag_winch (int dummy)
 {
-#ifdef PORT_NEEDS_CHANGE_SCREEN_SIZE
+    (void) dummy;
+#ifndef USE_NCURSES	/* don't do malloc in a signal handler */
     low_level_change_screen_size ();
-#endif /* PORT_NEEDS_CHANGE_SCREEN_SIZE */
+#endif
     winch_flag = 1;
 }
 
-void edit_adjust_size (Dlg_head * h);
-
-#ifdef PORT_NEEDS_CHANGE_SCREEN_SIZE
 static void
 low_level_change_screen_size (void)
 {
 #if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#if defined TIOCGWINSZ && !defined SCO_FLAVOR
+#if defined TIOCGWINSZ
     struct winsize winsz;
 
     winsz.ws_col = winsz.ws_row = 0;
@@ -770,7 +745,7 @@ low_level_change_screen_size (void)
     if (winsz.ws_col && winsz.ws_row){
 #if defined(NCURSES_VERSION) && defined(HAVE_RESIZETERM)
 	resizeterm(winsz.ws_row, winsz.ws_col);
-	clearok(stdscr,TRUE);	/* FIXME: sigwinch's should use a semaphore! */
+	clearok(stdscr,TRUE);	/* sigwinch's should use a semaphore! */
 #else
 	COLS = winsz.ws_col;
 	LINES = winsz.ws_row;
@@ -779,17 +754,17 @@ low_level_change_screen_size (void)
 	resize_subshell ();
 #endif
     }
-#endif /* TIOCGWINSZ && !SCO_FLAVOR */
+#endif /* TIOCGWINSZ */
 #endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
 }
 
-void change_screen_size (void)
+void
+change_screen_size (void)
 {
+    winch_flag = 0;
 #if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
-#if defined TIOCGWINSZ && !defined SCO_FLAVOR
-    extern Dlg_head *view_dlg;
-    extern Dlg_head *edit_dlg;
-    
+#if defined TIOCGWINSZ
+
 #ifndef NCURSES_VERSION
     mc_noraw_mode ();
     endwin ();
@@ -806,25 +781,20 @@ void change_screen_size (void)
     init_curses ();
 #endif
     setup_panels ();
-    if (current_dlg == view_dlg)
-	view_adjust_size (view_dlg);
-#ifdef USE_INTERNAL_EDIT
-    if (current_dlg == edit_dlg)
-	edit_adjust_size (edit_dlg);
-#endif
-    
+
+    /* Inform currently running dialog */
+    (*current_dlg->callback) (current_dlg, DLG_RESIZE, 0);
+
 #ifdef RESIZABLE_MENUBAR
-	menubar_arrange(the_menubar);
+    menubar_arrange (the_menubar);
 #endif
-		
+
     /* Now, force the redraw */
     do_refresh ();
     touchwin (stdscr);
-#endif /* TIOCGWINSZ && !SCO_FLAVOR */
-#endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
-    winch_flag = 0;
+#endif				/* TIOCGWINSZ */
+#endif				/* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
 }
-#endif /* HAVE_X */
 
 static int ok_to_refresh = 1;
 
@@ -836,22 +806,14 @@ void use_dash (int flag)
 	ok_to_refresh--;
 }
 
-void set_hintbar(char *str) 
+void set_hintbar(const char *str) 
 {
-#ifndef HAVE_X
-    if (xterm_flag && xterm_hintbar) {
-	fprintf (stdout, "\33]0;mc - %s\7", str);
-	fflush (stdout);
-    } else
-#endif
-    {
-        label_set_text (the_hint, str);
-        if (ok_to_refresh > 0)
-	    refresh();
-    }
+    label_set_text (the_hint, str);
+    if (ok_to_refresh > 0)
+        refresh();
 }
 
-void print_vfs_message (char *msg, ...)
+void print_vfs_message (const char *msg, ...)
 {
     va_list ap;
     char str [128];
@@ -865,25 +827,33 @@ void print_vfs_message (char *msg, ...)
 	return;
 
     if (!message_visible || !the_hint || !the_hint->widget.parent) {
+	int col, row;
 
 	if (!nice_rotating_dash || (ok_to_refresh <= 0))
 	    return;
+
+	/* Preserve current cursor position */
+	getyx (stdscr, row, col);
+
 	move (0, 0);
-	printw ("%-*s", COLS-1, str);
+	attrset (NORMAL_COLOR);
+	tty_printf ("%-*s", COLS-1, str);
+
+	/* Restore cursor position */
+	move(row, col);
 	mc_refresh ();
 	return;
     }
 
-    if (message_visible || (xterm_flag && xterm_hintbar)) {
+    if (message_visible) {
         set_hintbar(str);
     }
 }
 
 void rotate_dash (void)
 {
-#ifndef HAVE_X
-    static char rotating_dash [] = "|/-\\";
-    static int pos = 0;
+    static const char rotating_dash [] = "|/-\\";
+    static size_t pos = 0;
 
     if (!nice_rotating_dash || (ok_to_refresh <= 0))
 	return;
@@ -891,32 +861,13 @@ void rotate_dash (void)
     if (pos >= sizeof (rotating_dash)-1)
 	pos = 0;
     move (0, COLS-1);
+    attrset (NORMAL_COLOR);
     addch (rotating_dash [pos]);
     mc_refresh ();
     pos++;
-#endif
 }
 
-void remove_dash (void)
-{
-#ifndef HAVE_X
-    if (!nice_rotating_dash)
-	return;
-
-    /* Currently, it's much nicer with the CPU to do this instead of
-       calling do_refresh.
-
-       I should implement a routine called invalidate_region that would
-       send a draw message only to the affected views.  But for now
-       this is fine.
-    */
-    
-    move (0, COLS-1);
-    addch (' ');
-#endif    
-}
-
-char *get_nth_panel_name (int num)
+const char *get_nth_panel_name (int num)
 {
     static char buffer [BUF_SMALL];
     
@@ -945,14 +896,14 @@ void set_display_type (int num, int type)
 {
     int x, y, cols, lines;
     int    the_other;		/* Index to the other panel */
-    char   *file_name = 0;	/* For Quick view */
+    const char   *file_name = NULL;	/* For Quick view */
     Widget *new_widget, *old_widget;
     WPanel  *the_other_panel;
 
     x = y = cols = lines = 0;
     old_widget = 0;
     if (num >= MAX_VIEWS){
-	fprintf (stderr, "Could not allocate more that %d views\n", MAX_VIEWS);
+	fprintf (stderr, "Cannot allocate more that %d views\n", MAX_VIEWS);
 	abort ();
     }
 
@@ -1011,7 +962,7 @@ void set_display_type (int num, int type)
 	else
 	    file_name = "";
 	
-	view_init ((WView *) new_widget, 0, file_name, 0);
+	view_load ((WView *) new_widget, 0, file_name, 0);
 	break;
     }
     panels [num].type = type;
@@ -1020,18 +971,10 @@ void set_display_type (int num, int type)
     /* We set the same size the old widget had */
     widget_set_size ((Widget *) new_widget, y, x, lines, cols);
     
-    /* We wanna the new widget at the same position */
-    /* XView sets wcontainer to !0 <- Not XView, but we, when we create it */
-    /* Ok, the XView support code does it */
-    if (old_widget && old_widget->wcontainer){
-	new_widget->wcontainer = old_widget->wcontainer;
-	new_widget->area = old_widget->area;
-    }
-
     /* We use replace to keep the circular list of the dialog in the */
     /* same state.  Maybe we could just kill it and then replace it  */
     if (midnight_dlg && old_widget){
-	dlg_replace_widget (midnight_dlg, old_widget, panels [num].widget);
+	dlg_replace_widget (old_widget, panels [num].widget);
     }
     if (type == view_listing){
 	if (num == 0)
@@ -1049,11 +992,11 @@ void set_display_type (int num, int type)
      * - select left panel
      * - invoke menue left/tree
      * - as long as you stay in the left panel almost everything that uses
-     *   cpanel causes segfault, e.g. C-Enter, C-x c, ...
+     *   current_panel causes segfault, e.g. C-Enter, C-x c, ...
      */
 
     if (type != view_listing)
-	if (cpanel == (WPanel *) old_widget)
+	if (current_panel == (WPanel *) old_widget)
 	    current_panel = num == 0 ? right_panel : left_panel;
 }
 
@@ -1068,16 +1011,6 @@ void swap_panels ()
     WPanel *panel1, *panel2;
     int tmp_type;
     
-#if 0
-#ifdef HAVE_PORTABLE_TOKEN_PASTING
-#define panelswap(e) panel.##e = panel1->##e; panel1->##e = panel2->##e; panel2->##e = panel.##e;
-#define panelswapstr(e) strcpy (panel.##e, panel1->##e); strcpy (panel1->##e, panel2->##e); strcpy (panel2->##e, panel.##e);
-#else
-#define panelswap(e) panel./**/e = panel1->/**/e; panel1->/**/e = panel2->/**/e; panel2->/**/e = panel./**/e;
-#define panelswapstr(e) strcpy (panel./**/e, panel1->/**/e); strcpy (panel1->/**/e, panel2->/**/e); strcpy (panel2->/**/e, panel./**/e);
-#endif
-#endif
-
 #define panelswap(x) panel. x = panel1-> x; panel1-> x = panel2-> x; panel2-> x = panel. x;
 
 #define panelswapstr(e) strcpy (panel. e, panel1-> e); \
@@ -1102,14 +1035,15 @@ void swap_panels ()
 	
         panel1->searching = 0;
         panel2->searching = 0;
-        if (cpanel == panel1)
+        if (current_panel == panel1)
             current_panel = panel2;
         else
             current_panel = panel1;
-        if (midnight_dlg->current->widget == panels [0].widget)
-            dlg_select_widget (midnight_dlg, (void *) panels [1].widget);
-        else if (midnight_dlg->current->widget == panels [1].widget)
-            dlg_select_widget (midnight_dlg, (void *) panels [0].widget);
+
+        if (dlg_widget_active (panels[0].widget))
+            dlg_select_widget (panels [1].widget);
+        else if (dlg_widget_active (panels[1].widget))
+            dlg_select_widget (panels [0].widget);
     } else {
 	WPanel *tmp_panel;
 	
@@ -1159,14 +1093,15 @@ int get_display_type (int index)
     return panels [index].type;
 }
 
-Widget *get_panel_widget (int index)
+struct Widget *
+get_panel_widget (int index)
 {
-    return panels [index].widget;
+    return panels[index].widget;
 }
 
 int get_current_index (void)
 {
-    if (panels [0].widget == ((Widget *) cpanel))
+    if (panels [0].widget == ((Widget *) current_panel))
 	return 0;
     else
 	return 1;
@@ -1177,10 +1112,16 @@ int get_other_index (void)
     return !get_current_index ();
 }
 
+struct WPanel *
+get_other_panel (void)
+{
+    return (struct WPanel *) get_panel_widget (get_other_index ());
+}
+
 /* Returns the view type for the current panel/view */
 int get_current_type (void)
 {
-    if (panels [0].widget == (Widget *) cpanel)
+    if (panels [0].widget == (Widget *) current_panel)
 	return panels [0].type;
     else
 	return panels [1].type;
@@ -1189,7 +1130,7 @@ int get_current_type (void)
 /* Returns the view type of the unselected panel */
 int get_other_type (void)
 {
-    if (panels [0].widget == (Widget *) cpanel)
+    if (panels [0].widget == (Widget *) current_panel)
 	return panels [1].type;
     else
 	return panels [0].type;
