@@ -20,24 +20,19 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include <locale.h>
 
+#include <ctype.h>
+#include <errno.h>
+#include <locale.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
 #include <sys/types.h>
-
 #include <sys/stat.h>
+#include <unistd.h>
 
-#ifdef HAVE_UNISTD_H
-#   include <unistd.h>
-#endif
-
-#include <errno.h>
-#include <ctype.h>
-#include <signal.h>
-
-/* Program include files */
 #include "global.h"
 #include "tty.h"
 #include "dir.h"
@@ -180,7 +175,7 @@ int force_ugly_line_drawing = 0;
 int reset_hp_softkeys = 0;
 
 /* The prompt */
-char *prompt = NULL;
+const char *prompt = NULL;
 
 /* The widget where we draw the prompt */
 WLabel *the_prompt;
@@ -200,12 +195,18 @@ Mouse_Type use_mouse_p = MOUSE_NONE;
 /* If true, assume we are running on an xterm terminal */
 static int force_xterm = 0;
 
+/* If on, default for "No" in delete operations */
+int safe_delete = 0;
+
 /* Controls screen clearing before an exec */
 int clear_before_exec = 1;
 
 /* Asks for confirmation before deleting a file */
 int confirm_delete = 1;
 
+/* Asks for confirmation before deleting a hotlist entry */
+int confirm_directory_hotlist_delete = 1;
+
 /* Asks for confirmation before overwriting a file */
 int confirm_overwrite = 1;
 
@@ -492,12 +493,11 @@ quit_cmd (void)
     quit_cmd_internal (0);
 }
 
-int
+void
 quiet_quit_cmd (void)
 {
     print_last_revert = 1;
     quit_cmd_internal (1);
-    return quit;
 }
 
 /*
@@ -540,7 +540,6 @@ directory_history_add (struct WPanel *panel, const char *dir)
 /*
  *  If we moved to the parent directory move the selection pointer to
  *  the old directory name; If we leave VFS dir, remove FS specificator.
- *  Warn: This code spoils lwd string.
  *
  *  You do _NOT_ want to add any vfs aware code here. <pavel@ucw.cz>
  */
@@ -549,7 +548,9 @@ get_parent_dir_name (const char *cwd, const char *lwd)
 {
     const char *p;
     if (strlen (lwd) > strlen (cwd))
-	if ((p = strrchr (lwd, PATH_SEP)) && !strncmp (cwd, lwd, p - lwd)) {
+	if ((p = strrchr (lwd, PATH_SEP)) && !strncmp (cwd, lwd, p - lwd) &&
+	 (strlen (cwd) == p - lwd || (p == lwd && cwd[0] == PATH_SEP &&
+	  cwd[1] == '\0'))) {
 	    return (p + 1);
 	}
     return NULL;
@@ -691,21 +692,26 @@ directory_history_list (WPanel *panel)
 int
 load_prompt (int fd, void *unused)
 {
+    (void) fd;
+    (void) unused;
+
     if (!read_subshell_prompt ())
 	return 0;
 
     /* Don't actually change the prompt if it's invisible */
     if (current_dlg == midnight_dlg && command_prompt) {
+	char *tmp_prompt;
 	int prompt_len;
 
-	prompt = strip_ctrl_codes (subshell_prompt);
-	prompt_len = strlen (prompt);
+	tmp_prompt = strip_ctrl_codes (subshell_prompt);
+	prompt_len = strlen (tmp_prompt);
 
 	/* Check for prompts too big */
 	if (COLS > 8 && prompt_len > COLS - 8) {
-	    prompt[COLS - 8] = 0;
+	    tmp_prompt[COLS - 8] = '\0';
 	    prompt_len = COLS - 8;
 	}
+	prompt = tmp_prompt;
 	label_set_text (the_prompt, prompt);
 	winput_set_origin ((WInput *) cmdline, prompt_len,
 			   COLS - prompt_len);
@@ -724,11 +730,11 @@ load_prompt (int fd, void *unused)
 
 /* Used to emulate Lynx's entering leaving a directory with the arrow keys */
 int
-maybe_cd (int char_code, int move_up_dir)
+maybe_cd (int move_up_dir)
 {
     if (navigate_with_arrows) {
 	if (!cmdline->buffer[0]) {
-	    if (!move_up_dir) {
+	    if (move_up_dir) {
 		do_cd ("..", cd_exact);
 		return 1;
 	    }
@@ -961,7 +967,7 @@ menu_last_selected_cmd (void)
     the_menubar->active = 1;
     the_menubar->dropped = drop_menus;
     the_menubar->previous_widget = midnight_dlg->current->dlg_id;
-    dlg_select_widget (midnight_dlg, the_menubar);
+    dlg_select_widget (the_menubar);
 }
 
 static void
@@ -1207,10 +1213,10 @@ copy_other_tagged (void)
 static void
 init_labels (void)
 {
-    define_label (midnight_dlg, 1, _("Help"), help_cmd);
-    define_label (midnight_dlg, 2, _("Menu"), user_file_menu_cmd);
-    define_label (midnight_dlg, 9, _("PullDn"), menu_cmd);
-    define_label (midnight_dlg, 10, _("Quit"), quit_cmd);
+    buttonbar_set_label (midnight_dlg, 1, _("Help"), help_cmd);
+    buttonbar_set_label (midnight_dlg, 2, _("Menu"), user_file_menu_cmd);
+    buttonbar_set_label (midnight_dlg, 9, _("PullDn"), menu_cmd);
+    buttonbar_set_label (midnight_dlg, 10, _("Quit"), quit_cmd);
 }
 
 static const key_map ctl_x_map[] = {
@@ -1255,7 +1261,9 @@ nothing (void)
 
 static const key_map default_map[] = {
     {KEY_F (19), menu_last_selected_cmd},
-    {KEY_F (20), (key_callback) quiet_quit_cmd},
+    {KEY_F (20), quiet_quit_cmd},
+
+    {XCTRL ('@'), single_dirsize_cmd},
 
     /* Copy useful information to the command line */
     {ALT ('a'), copy_current_pathname},
@@ -1336,9 +1344,6 @@ static void
 init_xterm_support (void)
 {
     const char *termvalue;
-#ifdef HAVE_SLANG
-    char *term_entry;
-#endif
 
     termvalue = getenv ("TERM");
     if (!termvalue || !(*termvalue)) {
@@ -1347,17 +1352,7 @@ init_xterm_support (void)
     }
 
     /* Check mouse capabilities */
-#ifdef HAVE_SLANG
-    term_entry = SLtt_tigetent (const_cast(char *, termvalue));
-    xmouse_seq = SLtt_tigetstr ("Km", &term_entry);
-#else
-    xmouse_seq = tigetstr ("kmous");
-#endif
-
-    /* -1 means invalid capability, shouldn't happen, but let's make it 0 */
-    if ((long) xmouse_seq == -1) {
-	xmouse_seq = NULL;
-    }
+    xmouse_seq = tty_tgetstr ("Km");
 
     if (strcmp (termvalue, "cygwin") == 0) {
 	force_xterm = 1;
@@ -1366,6 +1361,7 @@ init_xterm_support (void)
 
     if (force_xterm || strncmp (termvalue, "xterm", 5) == 0
 	|| strncmp (termvalue, "rxvt", 4) == 0
+	|| strcmp (termvalue, "Eterm") == 0
 	|| strcmp (termvalue, "dtterm") == 0) {
 	xterm_flag = 1;
 
@@ -1376,7 +1372,14 @@ init_xterm_support (void)
 
 	/* Enable mouse unless explicitly disabled by --nomouse */
 	if (use_mouse_p != MOUSE_DISABLED) {
-	    use_mouse_p = MOUSE_XTERM;
+	    const char *color_term = getenv ("COLORTERM");
+	    if (strncmp (termvalue, "rxvt", 4) == 0 ||
+		(color_term != NULL && strncmp (color_term, "rxvt", 4) == 0) ||
+		strcmp (termvalue, "Eterm") == 0) {
+		    use_mouse_p = MOUSE_XTERM_NORMAL_TRACKING;
+	    } else {
+		use_mouse_p = MOUSE_XTERM_BUTTON_EVENT_TRACKING;
+	    }
 	}
     }
 }
@@ -1598,18 +1601,16 @@ midnight_callback (struct Dlg_head *h, dlg_msg_t msg, int parm)
     }
 }
 
-#define xtoolkit_panel_setup()
-
 /* Show current directory in the xterm title */
 void
 update_xterm_title_path (void)
 {
-    unsigned char *p, *s;
+    char *p, *s;
 
     if (xterm_flag && xterm_title) {
 	p = s = g_strdup (strip_home_and_password (current_panel->cwd));
 	do {
-	    if (!is_printable (*s))
+	    if (!is_printable ((unsigned char) *s))
 		*s = '?';
 	} while (*++s);
 	fprintf (stdout, "\33]0;mc - %s\7", p);
@@ -1651,8 +1652,6 @@ load_hint (int force)
 static void
 setup_panels_and_run_mc (void)
 {
-    xtoolkit_panel_setup ();
-
     add_widget (midnight_dlg, the_menubar);
     add_widget (midnight_dlg, get_panel_widget (0));
     add_widget (midnight_dlg, get_panel_widget (1));
@@ -1664,9 +1663,9 @@ setup_panels_and_run_mc (void)
     init_labels ();
 
     if (boot_current_is_left)
-	dlg_select_widget (midnight_dlg, get_panel_widget (0));
+	dlg_select_widget (get_panel_widget (0));
     else
-	dlg_select_widget (midnight_dlg, get_panel_widget (1));
+	dlg_select_widget (get_panel_widget (1));
 
     /* Run the Midnight Commander if no file was specified in the command line */
     run_dlg (midnight_dlg);
@@ -1810,9 +1809,10 @@ sigchld_handler_no_subshell (int sig)
 	    console_flag = 0;
 	}
     }
+    /* If we got here, some other child exited; ignore it */
 #endif				/* __linux__ */
 
-    /* If we got here, some other child exited; ignore it */
+    (void) sig;
 }
 
 static void
@@ -2095,7 +2095,7 @@ handle_args (int argc, char *argv[])
  */
 
 static int
-do_mc_filename_rename (const char *mc_dir, const char *o_name, const char *n_name)
+do_mc_filename_rename (const char *o_name, const char *n_name)
 {
     char *full_o_name = concat_dir_and_file (home_dir, o_name);
     char *full_n_name = g_strconcat (home_dir, MC_BASE, n_name, (char *) NULL);
@@ -2117,14 +2117,14 @@ compatibility_move_mc_files (void)
     if (stat (mc_dir, &s) && (errno == ENOENT)
 	&& (mkdir (mc_dir, 0777) != -1)) {
 
-	move = do_mc_filename_rename (mc_dir, ".mc.ini", "ini");
-	move += do_mc_filename_rename (mc_dir, ".mc.hot", "hotlist");
-	move += do_mc_filename_rename (mc_dir, ".mc.ext", "bindings");
-	move += do_mc_filename_rename (mc_dir, ".mc.menu", "menu");
-	move += do_mc_filename_rename (mc_dir, ".mc.bashrc", "bashrc");
-	move += do_mc_filename_rename (mc_dir, ".mc.inputrc", "inputrc");
-	move += do_mc_filename_rename (mc_dir, ".mc.tcshrc", "tcshrc");
-	move += do_mc_filename_rename (mc_dir, ".mc.tree", "Tree");
+	move = do_mc_filename_rename (".mc.ini", "ini");
+	move += do_mc_filename_rename (".mc.hot", "hotlist");
+	move += do_mc_filename_rename (".mc.ext", "bindings");
+	move += do_mc_filename_rename (".mc.menu", "menu");
+	move += do_mc_filename_rename (".mc.bashrc", "bashrc");
+	move += do_mc_filename_rename (".mc.inputrc", "inputrc");
+	move += do_mc_filename_rename (".mc.tcshrc", "tcshrc");
+	move += do_mc_filename_rename (".mc.tree", "Tree");
     }
     g_free (mc_dir);
     return move;
@@ -2159,12 +2159,12 @@ main (int argc, char *argv[])
     SLtt_Ignore_Beep = 1;
 #endif
 
+    handle_args (argc, argv);
+
     /* NOTE: This has to be called before slang_init or whatever routine
        calls any define_sequence */
     init_key ();
 
-    handle_args (argc, argv);
-
     /* Must be done before installing the SIGCHLD handler [[FIXME]] */
     handle_console (CONSOLE_INIT);
 
