@@ -1,14 +1,13 @@
-#ifndef __MENU_H
-#define __MENU_H
+#ifndef MC_MENU_H
+#define MC_MENU_H
 
-#include "dlg.h"
 #include "widget.h"
 
 typedef void (*callfn) (void);
 
 typedef struct {
     char first_letter;
-    char *text;
+    const char *text;
     int  hot_key;
     callfn call_back;
 } menu_entry;
@@ -37,14 +36,13 @@ typedef struct WMenu {
     int    selected;		/* Selected menu on the top bar */
     int    subsel;		/* Selected entry on the submenu */
     int    max_entry_len;	/* Cache value for the columns in a box */
-    int    previous_selection;	/* Selected widget before activating menu */
+    int    previous_widget;	/* Selected widget ID before activating menu */
 } WMenu;
 
-Menu  *create_menu     (char *name, menu_entry *entries, int count,
-			char *help_node);
+Menu  *create_menu     (const char *name, menu_entry *entries, int count,
+			const char *help_node);
 void   destroy_menu    (Menu *menu);
 WMenu *menubar_new     (int y, int x, int cols, Menu *menu[], int items);
 void   menubar_arrange (WMenu *menubar);
 
-#endif /* __MENU_H */
-
+#endif
