@@ -13,22 +13,23 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include <config.h>
-#include <sys/types.h>		/* Needed to include local .h files */
-#include <sys/stat.h>
-#include <sys/param.h>
+
 #include <string.h>
-#include "tty.h"
 #include <stdio.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "global.h"
+#include "tty.h"
 #include "dir.h"
 #include "panel.h"
 #include "main.h"
-#include "tree.h"
+#include "tree.h"		/* xtree_mode */
 #include "profile.h"
-#define WANT_WIDGETS
 #include "setup.h"
 #include "mouse.h"		/* To make view.h happy */
 #include "view.h"		/* For the externs */
@@ -39,49 +40,32 @@
 #include "menu.h"		/* menubar_visible declaration */
 #include "win.h"		/* lookup_key */
 #include "cmd.h"
-#include "file.h"
-#include "x.h"
-
-#ifdef HAVE_GNOME
-#    include "gdesktop.h"
-#endif
-
-#ifndef PORT_LIST_MODE_NAME
-#     define PORT_LIST_MODE_NAME "list_mode"
-#endif
-
-#ifndef PORT_LIST_MODE_DEFAULT
-#     define PORT_LIST_MODE_DEFAULT "full"
-#endif
-
-#include "../vfs/vfs.h"
-#ifdef USE_NETCODE
-#   include "../vfs/ftpfs.h"
-extern int use_netrc;
-extern int ftpfs_retry_seconds;
-extern int ftpfs_use_passive_connections;
-extern int ftpfs_use_unix_list_options;
-extern int ftpfs_first_cd_then_ls;
-#endif
-
-#ifdef HAVE_GNOME
-extern int tree_panel_visible;
-extern int we_can_afford_the_speed;
-int use_magic = 0;
-#endif
-
-/* "$Id: setup.c,v 1.38 2000/08/22 22:50:15 proskin Exp $" */
+#include "file.h"		/* safe_delete */
 
 #ifdef USE_VFS
-extern int vfs_timeout;
-extern int tar_gzipped_memlimit;
+#include "../vfs/gc.h"
 #endif
 
+#ifdef HAVE_CHARSET
+#include "charsets.h"
+#endif
+
+#ifdef USE_NETCODE
+#   include "../vfs/ftpfs.h"
+#   include "../vfs/fish.h"
+#endif
+
+#ifdef USE_INTERNAL_EDIT
+#   include "../edit/edit.h"
+#endif
+
+
 extern char *find_ignore_dirs;
 
 extern int num_history_items_recorded;
 
-char *profile_name;
+char *profile_name;		/* .mc/ini */
+char *global_profile_name;	/* mc.lib */
 
 char setup_color_string [4096];
 char term_color_string [4096];
@@ -99,7 +83,7 @@ int startup_right_mode;
 int saving_setup;
 
 static const struct {
-    char *key;
+    const char *key;
     sortfn *sort_type;
 } sort_names [] = {
     { "name",      (sortfn *) sort_name },
@@ -114,19 +98,18 @@ static const struct {
 };
 
 static const struct {
-    char *key;
+    const char *key;
     int  list_type;
 } list_types [] = {
     { "full",  list_full  },
     { "brief", list_brief },
     { "long",  list_long  },
     { "user",  list_user  },
-    { "icons", list_icons },
     { 0, 0 }
 };
 
 static const struct {
-    char *opt_name;
+    const char *opt_name;
     int  opt_type;
 } panel_types [] = {
     { "listing",   view_listing },
@@ -137,48 +120,34 @@ static const struct {
 };
 
 static const struct {
-    char *opt_name;
+    const char *opt_name;
     int *opt_addr;
 } layout [] = {
-#ifndef HAVE_GNOME
     { "equal_split", &equal_split },
     { "first_panel_size", &first_panel_size },
     { "message_visible", &message_visible },
     { "keybar_visible", &keybar_visible },
-    { "xterm_hintbar", &xterm_hintbar },
+    { "xterm_title", &xterm_title },
     { "output_lines", &output_lines },
-#endif
-    { "menubar_visible", &menubar_visible },
     { "command_prompt", &command_prompt },
+    { "menubar_visible", &menubar_visible },
     { "show_mini_info", &show_mini_info },
     { "permission_mode", &permission_mode },
     { "filetype_mode", &filetype_mode },
     { 0, 0 }
 };
 
-
-#undef SAVE_CHANGES_OUTSIDE_OPTIONS_MENU
-
-#ifdef SAVE_CHANGES_OUTSIDE_OPTIONS_MENU
-extern int preserve_uidgid;
-#endif
-
 static const struct {
-    char *opt_name;
+    const char *opt_name;
     int  *opt_addr;
-} options [] = {
+} int_options [] = {
     { "show_backups", &show_backups },
-#ifdef HAVE_GNOME
-    { "gnome_show_dot_files", &show_dot_files },
-#else
     { "show_dot_files", &show_dot_files },
-#endif
     { "verbose", &verbose },
     { "mark_moves_down", &mark_moves_down },
     { "pause_after_run", &pause_after_run },
     { "shell_patterns", &easy_patterns },
     { "auto_save_setup", &auto_save_setup },
-    { "align_extensions", &align_extensions },
     { "auto_menu", &auto_menu },
     { "use_internal_view", &use_internal_view },
     { "use_internal_edit", &use_internal_edit },
@@ -190,27 +159,26 @@ static const struct {
     { "confirm_overwrite", &confirm_overwrite },
     { "confirm_execute", &confirm_execute },
     { "confirm_exit", &confirm_exit },
-    { "safe_delete", &know_not_what_am_i_doing },
+    { "confirm_directory_hotlist_delete", &confirm_directory_hotlist_delete },
+    { "safe_delete", &safe_delete },
     { "mouse_repeat_rate", &mou_auto_repeat },
     { "double_click_speed", &double_click_speed },
+#ifndef HAVE_CHARSET
     { "eight_bit_clean", &eight_bit_clean },
     { "full_eight_bits", &full_eight_bits },
+#endif /* !HAVE_CHARSET */
     { "use_8th_bit_as_meta", &use_8th_bit_as_meta },
     { "confirm_view_dir", &confirm_view_dir },
     { "mouse_move_pages", &mouse_move_pages },
     { "mouse_move_pages_viewer", &mouse_move_pages_viewer },
     { "fast_refresh", &fast_refresh },
     { "navigate_with_arrows", &navigate_with_arrows },
-    { "advanced_chown", &advanced_chfns },
     { "drop_menus", &drop_menus },
     { "wrap_mode",  &global_wrap_mode},
     { "old_esc_mode", &old_esc_mode },
-#ifndef HAVE_GNOME
     { "cd_symlinks", &cd_symlinks },
-#endif
     { "show_all_if_ambiguous", &show_all_if_ambiguous },
-    { "have_fast_cpu", &have_fast_cpu },
-    { "iconify_on_exec", &iconify_on_exec },
+    { "max_dirt_limit", &max_dirt_limit },
     { "torben_fj_mode", &torben_fj_mode },
     { "use_file_to_guess_type", &use_file_to_check_type },
     { "alternate_plus_minus", &alternate_plus_minus },
@@ -220,16 +188,6 @@ static const struct {
     { "xtree_mode", &xtree_mode },
     { "num_history_items_recorded", &num_history_items_recorded },
     { "file_op_compute_totals", &file_op_compute_totals },
-#ifdef HAVE_GNOME
-    { "use_magic", &use_magic },
-#endif
-#ifdef SAVE_CHANGES_OUTSIDE_OPTIONS_MENU
-    { "dive_into_subdirs", &dive_into_subdirs },
-    { "preserve_uidgid", &preserve_uidgid },
-    /* What about the other two options in the copy dialog
-       (follow links, stable symlinks) -Norbert */
-    { "tree_navigation_flag", &tree_navigation_flag },
-#endif
 #ifdef USE_VFS
     { "vfs_timeout", &vfs_timeout },
 #ifdef USE_NETCODE
@@ -240,8 +198,9 @@ static const struct {
     { "ftpfs_use_passive_connections", &ftpfs_use_passive_connections },
     { "ftpfs_use_unix_list_options", &ftpfs_use_unix_list_options },
     { "ftpfs_first_cd_then_ls", &ftpfs_first_cd_then_ls },
-#endif
-#endif
+    { "fish_directory_timeout", &fish_directory_timeout },
+#endif /* USE_NETCODE */
+#endif /* USE_VFS */
 #ifdef USE_INTERNAL_EDIT
     { "editor_word_wrap_line_length", &option_word_wrap_line_length },
     { "editor_key_emulation", &edit_key_emulation },
@@ -251,55 +210,32 @@ static const struct {
     { "editor_backspace_through_tabs", &option_backspace_through_tabs },
     { "editor_fake_half_tabs", &option_fake_half_tabs },
     { "editor_option_save_mode", &option_save_mode },
-    { "editor_option_backup_ext_int", &option_backup_ext_int },
+    { "editor_option_save_position", &option_save_position },
     { "editor_option_auto_para_formatting", &option_auto_para_formatting },
     { "editor_option_typewriter_wrap", &option_typewriter_wrap },
     { "editor_edit_confirm_save", &edit_confirm_save },
     { "editor_syntax_highlighting", &option_syntax_highlighting },
-#endif
+#endif /* USE_INTERNAL_EDIT */
 
-#ifdef HAVE_GNOME
-    { "desktop_use_shaped_icons", &desktop_use_shaped_icons },
-    { "desktop_use_shaped_text", &desktop_use_shaped_text },
-    { "desktop_auto_placement", &desktop_auto_placement },
-    { "desktop_snap_icons", &desktop_snap_icons },
-    { "desktop_arr_r2l", &desktop_arr_r2l },
-    { "desktop_arr_b2t", &desktop_arr_b2t },
-    { "desktop_arr_rows", &desktop_arr_rows },
-    { "tree_panel_visible", &tree_panel_visible },
-    { "we_can_afford_the_speed", &we_can_afford_the_speed },
-
-    /* Keep the following in sync with panel.h */
-    { "column_width_brief_icon", &default_column_width[0] },
-    { "column_width_brief_filename", &default_column_width[1] },
-    { "column_width_detailed_icon", &default_column_width[2] },
-    { "column_width_detailed_filename", &default_column_width[3] },
-    { "column_width_detailed_size", &default_column_width[4] },
-    { "column_width_detailed_mtime", &default_column_width[5] },
-    { "column_width_custom_0", &default_column_width[6] },
-    { "column_width_custom_1", &default_column_width[7] },
-    { "column_width_custom_2", &default_column_width[8] },
-    { "column_width_custom_3", &default_column_width[9] },
-    { "column_width_custom_4", &default_column_width[10] },
-    { "column_width_custom_5", &default_column_width[11] },
-    { "column_width_custom_6", &default_column_width[12] },
-    { "column_width_custom_7", &default_column_width[13] },
-    { "column_width_custom_8", &default_column_width[14] },
-    { "column_width_custom_9", &default_column_width[15] },
-    { "column_width_custom_10", &default_column_width[16] },
-    { "column_width_custom_11", &default_column_width[17] },
-    { "column_width_custom_12", &default_column_width[18] },
-    { "column_width_custom_13", &default_column_width[19] },
-    { "column_width_custom_14", &default_column_width[20] },
-#else
     { "nice_rotating_dash", &nice_rotating_dash },
     { "horizontal_split",   &horizontal_split },
-#endif
+    { "mcview_remember_file_position", &mcview_remember_file_position },
     { 0, 0 }
 };
 
+static const struct {
+    const char *opt_name;
+    char **opt_addr;
+    const char *opt_defval;
+} str_options [] = {
+#ifdef USE_INTERNAL_EDIT
+    { "editor_backup_extension", &option_backup_ext, "~" },
+#endif
+    { NULL, NULL }
+};
+
 void
-panel_save_setup (WPanel *panel, char *section)
+panel_save_setup (struct WPanel *panel, const char *section)
 {
     char buffer [BUF_TINY];
     int  i;
@@ -317,7 +253,7 @@ panel_save_setup (WPanel *panel, char *section)
 
     for (i = 0; list_types [i].key; i++)
 	if (list_types [i].list_type == panel->list_type){
-	    save_string (section, PORT_LIST_MODE_NAME, list_types [i].key, profile_name);
+	    save_string (section, "list_mode", list_types [i].key, profile_name);
 	    break;
 	}
 
@@ -362,18 +298,19 @@ save_configure (void)
     profile = concat_dir_and_file (home_dir, PROFILE_NAME);
 
     /* Save integer options */
-    for (i = 0; options [i].opt_name; i++)
-	set_int (profile, options [i].opt_name, *options [i].opt_addr);
+    for (i = 0; int_options[i].opt_name; i++)
+	set_int (profile, int_options[i].opt_name, *int_options[i].opt_addr);
 
-#ifdef HAVE_GNOME
-    save_string (app_text, "default_user_format", default_user_format, profile_name);
-#endif
+    /* Save string options */
+    for (i = 0; str_options[i].opt_name != NULL; i++)
+	set_config_string (profile, str_options[i].opt_name,
+	    *str_options[i].opt_addr);
 
     g_free (profile);
 }
 
 static void
-panel_save_type (char *section, int type)
+panel_save_type (const char *section, int type)
 {
     int i;
 
@@ -385,9 +322,8 @@ panel_save_type (char *section, int type)
 	}
 }
 
-#ifndef PORT_HAS_SAVE_PANEL_TYPES
-void
-save_panel_types ()
+static void
+save_panel_types (void)
 {
     int type;
 
@@ -400,49 +336,49 @@ save_panel_types ()
     if (type == view_listing)
 	panel_save_setup (right_panel, right_panel->panel_name);
 }
-#endif
 
 void
 save_setup (void)
 {
     char *profile;
-#ifdef USE_VFS
-#ifdef USE_NETCODE
-    extern char *ftpfs_anonymous_passwd;
-    extern char *ftpfs_proxy_host;
-#endif
-#endif
+
     saving_setup = 1;
     profile = concat_dir_and_file (home_dir, PROFILE_NAME);
 
-    save_layout ();
     save_configure ();
+
+    save_layout ();
     save_string ("Dirs", "other_dir",
 			       get_other_type () == view_listing
-			       ? opanel->cwd : ".", profile);
-    if (get_current_panel () != NULL)
+			       ? other_panel->cwd : ".", profile);
+    if (current_panel != NULL)
 	    WritePrivateProfileString ("Dirs", "current_is_left",
 				       get_current_index () == 0 ? "1" : "0", profile);
     save_hotlist ();
+
     save_panelize ();
     save_panel_types ();
 /*     directory_history_save (); */
 
-#ifdef USE_VFS
-#ifdef USE_NETCODE
+#if defined(USE_VFS) && defined (USE_NETCODE)
     WritePrivateProfileString ("Misc", "ftpfs_password",
 			       ftpfs_anonymous_passwd, profile);
     if (ftpfs_proxy_host)
 	WritePrivateProfileString ("Misc", "ftp_proxy_host",
 				   ftpfs_proxy_host, profile);
-#endif
-#endif
+#endif /* USE_VFS && USE_NETCODE */
+
+#ifdef HAVE_CHARSET
+    save_string( "Misc", "display_codepage",
+    		 get_codepage_id( display_codepage ), profile_name );
+#endif /* HAVE_CHARSET */
+
     g_free (profile);
     saving_setup = 0;
 }
 
 void
-panel_load_setup (WPanel *panel, char *section)
+panel_load_setup (WPanel *panel, const char *section)
 {
     int i;
     char buffer [BUF_TINY];
@@ -460,28 +396,21 @@ panel_load_setup (WPanel *panel, char *section)
 	}
 
     /* Load the listing mode */
-    load_string (section, PORT_LIST_MODE_NAME, PORT_LIST_MODE_DEFAULT, buffer, sizeof (buffer));
+    load_string (section, "list_mode", "full", buffer, sizeof (buffer));
     panel->list_type = list_full;
     for (i = 0; list_types [i].key; i++)
 	if ( g_strcasecmp (list_types [i].key, buffer) == 0){
 	    panel->list_type = list_types [i].list_type;
 	    break;
 	}
-#ifndef PORT_HAS_ICON_VIEW
-    if (panel->list_type == list_icons)
-        panel->list_type = list_full;
-#endif
+
     /* User formats */
-    if (panel->user_format){
-	g_free (panel->user_format);
-	panel->user_format = 0;
-    }
+    g_free (panel->user_format);
     panel->user_format = g_strdup (get_profile_string (section, "user_format",
 						     DEFAULT_USER_FORMAT,
 						     profile_name));
     for (i = 0; i < LIST_TYPES; i++){
-        if (panel->user_status_format [i])
-	    g_free (panel->user_status_format [i]);
+	g_free (panel->user_status_format [i]);
 	g_snprintf (buffer, sizeof (buffer), "user_status%d", i);
 	panel->user_status_format [i] =
 	    g_strdup (get_profile_string (section, buffer,
@@ -505,7 +434,7 @@ load_layout (char *profile_name)
 }
 
 static int
-load_mode (char *section)
+load_mode (const char *section)
 {
     char buffer [20];
     int  i;
@@ -524,8 +453,9 @@ load_mode (char *section)
     return mode;
 }
 
+#ifdef USE_NETCODE
 static char *
-do_load_string (char *s, char *ss, char *def)
+do_load_string (const char *s, const char *ss, const char *def)
 {
     char *buffer = g_malloc (BUF_SMALL);
     char *p;
@@ -536,28 +466,26 @@ do_load_string (char *s, char *ss, char *def)
     g_free (buffer);
     return p;
 }
+#endif /* !USE_NETCODE */
 
 char *
 setup_init (void)
 {
-    char *buffer;
     char   *profile;
     char   *inifile;
 
     if (profile_name)
 	    return profile_name;
 
-    buffer = concat_dir_and_file (home_dir, PROFILE_NAME);
-    inifile = concat_dir_and_file (mc_home, "mc.ini");
-    if (exist_file (buffer)){
-	profile = buffer;
-    } else if (exist_file (inifile)){
-	profile = g_strdup (inifile);
-	g_free (buffer);
-    } else {
-	profile = buffer;
+    profile = concat_dir_and_file (home_dir, PROFILE_NAME);
+    if (!exist_file (profile)){
+	inifile = concat_dir_and_file (mc_home, "mc.ini");
+	if (exist_file (inifile)){
+	    g_free (profile);
+	    profile = inifile;
+	} else
+	    g_free (inifile);
     }
-    g_free (inifile);
 
     profile_name = profile;
 
@@ -568,22 +496,23 @@ void
 load_setup (void)
 {
     char *profile;
-
     int    i;
-#ifdef USE_NETCODE
-    extern char *ftpfs_proxy_host;
-#endif
 
     profile = setup_init ();
-    /* Load integer boolean options */
-    for (i = 0; options [i].opt_name; i++)
-	*options [i].opt_addr =
-	    get_int (profile, options [i].opt_name, *options [i].opt_addr);
 
-#ifdef HAVE_GNOME
-    g_free (default_user_format);
-    default_user_format = do_load_string (app_text, "default_user_format", DEFAULT_USER_FORMAT);
-#endif
+    /* mc.lib is common for all users, but has priority lower than
+       ~/.mc/ini.  FIXME: it's only used for keys and treestore now */
+    global_profile_name = concat_dir_and_file (mc_home, "mc.lib");
+
+    /* Load integer boolean options */
+    for (i = 0; int_options[i].opt_name; i++)
+	*int_options[i].opt_addr =
+	    get_int (profile, int_options[i].opt_name, *int_options[i].opt_addr);
+
+    /* Load string options */
+    for (i = 0; str_options[i].opt_name != NULL; i++)
+	*str_options[i].opt_addr = get_config_string (profile,
+	    str_options[i].opt_name, str_options[i].opt_defval);
 
     load_layout (profile);
 
@@ -607,16 +536,18 @@ load_setup (void)
 	else
 	    g_free (buffer);
     }
+
+    boot_current_is_left =
+	GetPrivateProfileInt ("Dirs", "current_is_left", 1, profile);
+
 #ifdef USE_NETCODE
     ftpfs_proxy_host = do_load_string ("Misc", "ftp_proxy_host", "gate");
 #endif
-    boot_current_is_left =
-	GetPrivateProfileInt ("Dirs", "current_is_left", 1, profile);
 
     load_string ("Misc", "find_ignore_dirs", "", setup_color_string,
 		 sizeof (setup_color_string));
     if (setup_color_string [0])
-	find_ignore_dirs = g_strconcat (":", setup_color_string, ":", NULL);
+	find_ignore_dirs = g_strconcat (":", setup_color_string, ":", (char *) NULL);
 
     /* The default color and the terminal dependent color */
     load_string ("Colors", "base_color", "", setup_color_string,
@@ -631,15 +562,24 @@ load_setup (void)
     /* Remove the temporal entries */
     profile_clean_section ("Temporal:New Left Panel", profile_name);
     profile_clean_section ("Temporal:New Right Panel", profile_name);
-#ifdef USE_VFS
-#ifdef USE_NETCODE
+#if defined(USE_VFS) && defined (USE_NETCODE)
     ftpfs_init_passwd ();
-#endif
-#endif
+#endif /* USE_VFS && USE_NETCODE */
+
+#ifdef HAVE_CHARSET
+    if ( load_codepages_list() > 0 ) {
+	char cpname[128];
+	load_string( "Misc", "display_codepage", "",
+		     cpname, sizeof(cpname) );
+	if ( cpname[0] != '\0' )
+	    display_codepage = get_codepage_index( cpname );
+    }
+
+    init_translation_table( source_codepage, display_codepage );
+#endif /* HAVE_CHARSET */
 }
 
-#ifdef USE_VFS
-#ifdef USE_NETCODE
+#if defined(USE_VFS) && defined (USE_NETCODE)
 char *
 load_anon_passwd ()
 {
@@ -651,19 +591,19 @@ load_anon_passwd ()
     else
 	return 0;
 }
-#endif
-#endif
+#endif /* USE_VFS && USE_NETCODE */
 
 void done_setup (void)
 {
     g_free (profile_name);
+    g_free (global_profile_name);
     done_hotlist ();
     done_panelize ();
 /*    directory_history_free (); */
 }
 
 static void
-load_keys_from_section (char *terminal, char *profile_name)
+load_keys_from_section (const char *terminal, const char *profile_name)
 {
     char *section_name;
     void *profile_keys;
@@ -673,35 +613,38 @@ load_keys_from_section (char *terminal, char *profile_name)
     if (!terminal)
 	return;
 
-    section_name = g_strconcat ("terminal:", terminal, NULL);
+    section_name = g_strconcat ("terminal:", terminal, (char *) NULL);
     profile_keys = profile_init_iterator (section_name, profile_name);
-    if (!profile_keys){
-	g_free (section_name);
-	return;
-    }
-
+    g_free (section_name);
     while (profile_keys){
 	profile_keys = profile_iterator_next (profile_keys, &key, &value);
+
+	/* copy=other causes all keys from [terminal:other] to be loaded.  */
+	if (g_strcasecmp (key, "copy") == 0) {
+	    load_keys_from_section (value, profile_name);
+	    continue;
+	}
+
 	key_code = lookup_key (key);
-	valcopy = convert_controls (value);
-	if (key_code)
+	if (key_code){
+	    valcopy = convert_controls (value);
 	    define_sequence (key_code, valcopy, MCKEY_NOACTION);
-	g_free (valcopy);
+	    g_free (valcopy);
+	}
     }
-    g_free (section_name);
-    return;
 }
 
 void load_key_defs (void)
 {
-    char *libfile = concat_dir_and_file (mc_home, "mc.lib");
-    load_keys_from_section (getenv ("TERM"), profile_name);
+    /*
+     * Load keys from mc.lib before ~/.mc/ini, so that the user
+     * definitions override global settings.
+     */
+    load_keys_from_section ("general", global_profile_name);
+    load_keys_from_section (getenv ("TERM"), global_profile_name);
     load_keys_from_section ("general", profile_name);
-
-    load_keys_from_section (getenv ("TERM"), libfile);
-    load_keys_from_section ("general", libfile);
+    load_keys_from_section (getenv ("TERM"), profile_name);
 
     /* We don't want a huge database loaded in core */
-    free_profile_name (libfile);
-    g_free (libfile);
+    free_profile_name (global_profile_name);
 }
