@@ -17,7 +17,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
    This module has been converted to be a widget.
 
@@ -27,16 +27,17 @@
 
    */
 #include <config.h>
-#include <string.h>
+
 #include <errno.h>
 #include <stdio.h>
+#include <string.h>
 
 #include "global.h"
 #include "tty.h"
 #include "color.h"
 #include "wtools.h"	/* message() */
 #include "dir.h"
-#include "dlg.h"
+#include "dialog.h"
 #include "widget.h"
 #include "panel.h"
 #include "mouse.h"
@@ -45,19 +46,22 @@
 #include "help.h"
 #include "key.h"	/* For mi_getch() */
 #include "tree.h"
+#include "treestore.h"
 #include "cmd.h"
 
+#define tlines(t) (t->is_panel ? t->widget.lines-2 - (show_mini_info ? 2 : 0) : t->widget.lines)
+
 extern int command_prompt;
 
 /* Use the color of the parent widget for the unselected entries */
-#define TREE_NORMALC NORMALC
+#define TREE_NORMALC(h) (DLG_NORMALC (h))
 
 /* Specifies the display mode: 1d or 2d */
 static int tree_navigation_flag;
 
 struct WTree {
     Widget widget;
-    TreeStore *store;
+    struct TreeStore *store;
     tree_entry *selected_ptr;	/* The selected directory */
     char search_buffer[256];	/* Current search string */
     tree_entry **tree_shown;	/* Entries currently on screen */
@@ -70,8 +74,7 @@ struct WTree {
 
 /* Forwards */
 static void save_tree (WTree *tree);
-static void tree_rescan_cmd (WTree *tree);
-static int tree_callback (WTree *tree, int msg, int par);
+static void tree_rescan_cmd (WTree *);
 
 static tree_entry *back_ptr (tree_entry *ptr, int *count)
 {
@@ -97,63 +100,6 @@ static tree_entry *forw_ptr (tree_entry *ptr, int *count)
     return ptr;
 }
 
-#if 0
-/* Add a directory to the list of directories */
-static tree_entry *tree_add_entry (WTree *tree, char *name)
-{
-    if (!tree)
-	return 0;
-
-    return tree_store_add_entry (name);
-}
-
-/* Append a directory to the list of directories */
-static tree_entry *tree_append_entry (WTree *tree, char *name)
-{
-    tree_entry *current, *new;
-    int i, len;
-    int submask = 0;
-
-    /* We assume the directory is not yet in the list */
-
-    new = g_new (tree_entry, 1);
-    if (!tree->store->tree_first){
-        /* Empty list */
-        tree->store->tree_first = new;
-        new->prev = NULL;
-    } else {
-        tree->tree_last->next = new;
-        new->prev = tree->tree_last;
-    }
-    new->next = NULL;
-    tree->store->tree_last = new;
-
-    /* Calculate attributes */
-    new->name = g_strdup (name);
-    len = strlen (new->name);
-    new->sublevel = 0;
-    for (i = 0; i < len; i++)
-	if (new->name [i] == PATH_SEP){
-	    new->sublevel++;
-	    new->subname = new->name + i + 1;
-	}
-    submask = 1 << new->sublevel;
-    submask &= (2 << new->sublevel) - 1;
-    new->submask = submask;
-    new->mark = 0;
-
-    /* Correct the submasks of the previous entries */
-    current = new->prev;
-    while (current && current->sublevel > new->sublevel){
-	current->submask |= 1 << new->sublevel;
-	current = current->prev;
-    }
-
-    /* The entry has now been appended */
-    return new;
-}
-#endif
-
 static void
 remove_callback (tree_entry *entry, void *data)
 {
@@ -169,6 +115,7 @@ remove_callback (tree_entry *entry, void *data)
 
 static void tree_remove_entry (WTree *tree, char *name)
 {
+    (void) tree;
     tree_store_remove_entry (name);
 }
 
@@ -177,10 +124,8 @@ static void tree_destroy (WTree *tree)
     tree_store_remove_entry_remove_hook (remove_callback);
     save_tree (tree);
 
-    if (tree->tree_shown){
-	g_free (tree->tree_shown);
-	tree->tree_shown = 0;
-    }
+    g_free (tree->tree_shown);
+    tree->tree_shown = 0;
     tree->selected_ptr = NULL;
 }
 
@@ -198,12 +143,12 @@ static void save_tree (WTree *tree)
 {
     int error;
 
+    (void) tree;
     error = tree_store_save ();
 
     if (error){
 	fprintf (stderr, _("Cannot open the %s file for writing:\n%s\n"), MC_TREE,
 		 unix_error_string (error));
-	return;
     }
 }
 
@@ -226,16 +171,16 @@ static void tree_show_mini_info (WTree *tree, int tree_lines, int tree_cols)
 
     if (tree->searching){
 	/* Show search string */
-	attrset (TREE_NORMALC);
-	attrset (FOCUSC);
+	attrset (TREE_NORMALC (h));
+	attrset (DLG_FOCUSC (h));
 	addch (PATH_SEP);
 
-	addstr (name_trunc (tree->search_buffer, tree_cols-2));
+	addstr ((char *) name_trunc (tree->search_buffer, tree_cols-2));
 	addch (' ');
-	attrset (FOCUSC);
+	attrset (DLG_FOCUSC (h));
     } else {
 	/* Show full name of selected directory */
-	addstr (name_trunc (tree->selected_ptr->name, tree_cols));
+	addstr ((char *) name_trunc (tree->selected_ptr->name, tree_cols));
     }
 }
 
@@ -252,15 +197,14 @@ static void show_tree (WTree *tree)
     tree_lines = tlines (tree);
     tree_cols  = tree->widget.cols;
 
-    attrset (TREE_NORMALC);
+    attrset (TREE_NORMALC (h));
     widget_move ((Widget*)tree, y, x);
     if (tree->is_panel){
 	tree_cols  -= 2;
 	x = y = 1;
     }
 
-    if (tree->tree_shown)
-	g_free (tree->tree_shown);
+    g_free (tree->tree_shown);
     tree->tree_shown = g_new (tree_entry*, tree_lines);
 
     for (i = 0; i < tree_lines; i++)
@@ -322,7 +266,7 @@ static void show_tree (WTree *tree)
 	    }
 
 	    /* Show full name */
-	    addstr (name_trunc (current->name, tree_cols - 6));
+	    addstr ((char *) name_trunc (current->name, tree_cols - 6));
 	} else{
 	    /* Sub level directory */
 
@@ -356,13 +300,13 @@ static void show_tree (WTree *tree)
 
 	    /* Show sub-name */
 	    addch (' ');
-	    addstr (name_trunc (current->subname,
+	    addstr ((char *) name_trunc (current->subname,
 				tree_cols - 2 - 4 - 3 * j));
 	}
 	addch (' ');
 
 	/* Return to normal color */
-	attrset (TREE_NORMALC);
+	attrset (TREE_NORMALC (h));
 
 	/* Calculate the next value for current */
 	current = current->next;
@@ -500,7 +444,7 @@ static void tree_move_to_bottom (WTree *tree)
     tree->topdiff = tlines (tree) - 3 - 1;
 }
 
-void tree_chdir (WTree *tree, char *dir)
+void tree_chdir (WTree *tree, const char *dir)
 {
     tree_entry *current;
 
@@ -512,13 +456,14 @@ void tree_chdir (WTree *tree, char *dir)
 }
 
 void
-sync_tree (char *path)
+sync_tree (const char *path)
 {
     tree_chdir (the_tree, path);
 }
 
 /* Handle mouse click */
-static void tree_event (WTree *tree, int y)
+static void
+tree_event (WTree *tree, int y)
 {
     if (tree->tree_shown [y]){
 	tree->selected_ptr = tree->tree_shown [y];
@@ -538,8 +483,11 @@ static void maybe_chdir (WTree *tree)
 }
 
 /* Mouse callback */
-static int event_callback (Gpm_Event *event, WTree *tree)
+static int
+event_callback (Gpm_Event *event, void *data)
 {
+    WTree *tree = data;
+
     if (!(event->type & GPM_UP))
 	return MOU_NORMAL;
 
@@ -597,10 +545,10 @@ static int search_tree (WTree *tree, char *text)
 
 static void tree_do_search (WTree *tree, int key)
 {
-    int l;
+    size_t l;
 
     l = strlen (tree->search_buffer);
-    if (l && (key == 8 || key == 0177 || key == KEY_BACKSPACE))
+    if (l && (key == KEY_BACKSPACE))
 	tree->search_buffer [--l] = 0;
     else {
 	if (key && l < sizeof (tree->search_buffer)){
@@ -617,7 +565,8 @@ static void tree_do_search (WTree *tree, int key)
     maybe_chdir (tree);
 }
 
-static void tree_rescan_cmd (WTree *tree)
+static void
+tree_rescan_cmd (WTree *tree)
 {
     char old_dir [MC_MAXPATHLEN];
 
@@ -629,14 +578,15 @@ static void tree_rescan_cmd (WTree *tree)
     mc_chdir (old_dir);
 }
 
-static int tree_forget_cmd (WTree *tree)
+static void
+tree_forget_cmd (void *data)
 {
+    WTree *tree = data;
     if (tree->selected_ptr)
 	tree_remove_entry (tree, tree->selected_ptr->name);
-    return 1;
 }
 
-static void tree_copy (WTree *tree, char *default_dest)
+static void tree_copy (WTree *tree, const char *default_dest)
 {
     char   *dest;
     off_t  count = 0;
@@ -657,26 +607,28 @@ static void tree_copy (WTree *tree, char *default_dest)
 	return;
     }
 
-    ctx = file_op_context_new ();
-    file_op_context_create_ui (ctx, OP_COPY, FALSE);
+    ctx = file_op_context_new (OP_COPY);
+    file_op_context_create_ui (ctx, FALSE);
     copy_dir_dir (ctx, tree->selected_ptr->name, dest, 1, 0, 0, 0, &count, &bytes);
     file_op_context_destroy (ctx);
 
     g_free (dest);
 }
 
-static void tree_help_cmd (void)
+static void
+tree_help_cmd (void)
 {
     interactive_display (NULL,  "[Directory Tree]");
 }
 
-static int tree_copy_cmd (WTree *tree)
+static void
+tree_copy_cmd (void *data)
 {
+    WTree *tree = data;
     tree_copy (tree, "");
-    return 1;
 }
 
-static void tree_move (WTree *tree, char *default_dest)
+static void tree_move (WTree *tree, const char *default_dest)
 {
     char   *dest;
     struct stat buf;
@@ -708,83 +660,80 @@ static void tree_move (WTree *tree, char *default_dest)
 	return;
     }
 
-    ctx = file_op_context_new ();
-    file_op_context_create_ui (ctx, OP_MOVE, FALSE);
+    ctx = file_op_context_new (OP_MOVE);
+    file_op_context_create_ui (ctx, FALSE);
     move_dir_dir (ctx, tree->selected_ptr->name, dest, &count, &bytes);
     file_op_context_destroy (ctx);
 
     g_free (dest);
 }
 
-static int
-tree_move_cmd (WTree *tree)
+static void
+tree_move_cmd (void *data)
 {
+    WTree *tree = data;
     tree_move (tree, "");
-    return 1;
 }
 
 #if 0
-static int
+static void
 tree_mkdir_cmd (WTree *tree)
 {
     char old_dir [MC_MAXPATHLEN];
 
     if (!tree->selected_ptr)
-	return 0;
+	return;
     if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
-	return 0;
+	return;
     if (chdir (tree->selected_ptr->name))
-	return 0;
+	return;
     /* FIXME
     mkdir_cmd (tree);
     */
     tree_rescan_cmd (tree);
     chdir (old_dir);
-    return 1;
 }
 #endif
 
 static void
 tree_rmdir_cmd (WTree *tree)
 {
-    char old_dir [MC_MAXPATHLEN];
     off_t count = 0;
     double bytes = 0;
     FileOpContext *ctx;
 
-    if (tree->selected_ptr){
-	if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
-	    return;
-	if (mc_chdir (PATH_SEP_STR))
-	    return;
-	if (confirm_delete){
-	    char *buf;
-	    int result;
-
-	    buf = g_strdup_printf (_("  Delete %s?  "), tree->selected_ptr->name);
-	    result = query_dialog (_(" Delete "), buf, 3, 2, _("&Yes"), _("&No"));
-	    g_free (buf);
-	    if (result != 0){
-		return;
-	    }
-	}
-
-	ctx = file_op_context_new ();
-	file_op_context_create_ui (ctx, OP_DELETE, FALSE);
-	if (erase_dir (ctx, tree->selected_ptr->name, &count, &bytes) == FILE_CONT)
-	    tree_forget_cmd (tree);
-	file_op_context_destroy (ctx);
-	mc_chdir (old_dir);
-	return;
-    } else
+    if (!tree->selected_ptr)
 	return;
+
+    if (confirm_delete) {
+	char *buf;
+	int result;
+
+	buf =
+	    g_strdup_printf (_("  Delete %s?  "),
+			     tree->selected_ptr->name);
+	result =
+	    query_dialog (_(" Delete "), buf, 3, 2, _("&Yes"), _("&No"));
+	g_free (buf);
+	if (result != 0)
+	    return;
+    }
+
+    ctx = file_op_context_new (OP_DELETE);
+    file_op_context_create_ui (ctx, FALSE);
+    if (erase_dir (ctx, tree->selected_ptr->name, &count, &bytes) ==
+	FILE_CONT)
+	tree_forget_cmd (tree);
+    file_op_context_destroy (ctx);
 }
 
 static void set_navig_label (WTree *tree);
 
 static void
-tree_toggle_navig (WTree *tree)
+tree_toggle_navig (void *data)
 {
+    WTree *tree = data;
+    /* FIXME: invalid use of boolean variable */
     tree_navigation_flag = 1 - tree_navigation_flag;
     set_navig_label (tree);
 }
@@ -792,9 +741,9 @@ tree_toggle_navig (WTree *tree)
 static void
 set_navig_label (WTree *tree)
 {
-    define_label_data (tree->widget.parent, 4,
+    buttonbar_set_label_data (tree->widget.parent, 4,
 		       tree_navigation_flag ? _("Static") : _("Dynamc"),
-		       (buttonbarfn) tree_toggle_navig, tree);
+		       tree_toggle_navig, tree);
 }
 
 static void
@@ -879,8 +828,7 @@ chdir_sel (WTree *tree)
     }
     change_panel ();
     if (do_cd (tree->selected_ptr->name, cd_exact)) {
-	paint_panel (cpanel);
-	select_item (cpanel);
+	select_item (current_panel);
     } else {
 	message (1, MSG_ERROR, _(" Cannot chdir to \"%s\" \n %s "),
 		 tree->selected_ptr->name, unix_error_string (errno));
@@ -918,7 +866,13 @@ tree_start_search (WTree *tree)
     }
 }
 
-static const key_map tree_keymap [] = {
+typedef void (*tree_key_action) (WTree *);
+typedef struct {
+    int key_code;
+    tree_key_action fn;
+} tree_key_map;
+
+static const tree_key_map tree_keymap [] = {
     { XCTRL('n'), move_down    },
     { XCTRL('p'), move_up      },
     { KEY_DOWN,   move_down    },
@@ -926,9 +880,11 @@ static const key_map tree_keymap [] = {
     { '\n',       chdir_sel    },
     { KEY_ENTER,  chdir_sel    },
     { KEY_HOME,   move_home    },
-    { KEY_C1,     move_end     },
-    { KEY_END,    move_end     },
     { KEY_A1,     move_home    },
+    { ALT ('<'),  move_home    },
+    { KEY_END,    move_end     },
+    { KEY_C1,     move_end     },
+    { ALT ('>'),  move_end     },
     { KEY_NPAGE,  move_nextp   },
     { KEY_PPAGE,  move_prevp   },
     { XCTRL('v'), move_nextp   },
@@ -942,7 +898,7 @@ static const key_map tree_keymap [] = {
     { 0, 0 }
     };
 
-static inline int
+static inline cb_ret_t
 tree_key (WTree *tree, int key)
 {
     int i;
@@ -953,45 +909,46 @@ tree_key (WTree *tree, int key)
 	        tree->searching = 0;
 	    (*tree_keymap [i].fn)(tree);
 	    show_tree (tree);
-	    return 1;
+	    return MSG_HANDLED;
 	}
     }
 
     /* We do not want to use them if we do not need to */
     /* Input line may want to take the motion key event */
     if (key == KEY_LEFT)
-	return move_left (tree);
+	return move_left (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
 
     if (key == KEY_RIGHT)
-	return move_right (tree);
+	return move_right (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
 
     if (is_abort_char (key)) {
 	if (tree->is_panel) {
 	    tree->searching = 0;
 	    show_tree (tree);
-	    return 1;  /* eat abort char */
+	    return MSG_HANDLED;  /* eat abort char */
 	}
-	return 0;  /* modal tree dialog: let upper layer see the
-		      abort character and close the dialog */
+	/* modal tree dialog: let upper layer see the
+	   abort character and close the dialog */
+	return MSG_NOT_HANDLED;
     }
 
     /* Do not eat characters not meant for the tree below ' ' (e.g. C-l). */
-    if ((key >= ' '&& key <= 255) || key == 8 || key == KEY_BACKSPACE) {
+    if ((key >= ' ' && key <= 255) || key == KEY_BACKSPACE) {
 	if (tree->searching){
 	    tree_do_search (tree, key);
 	    show_tree (tree);
-	    return 1;
+	    return MSG_HANDLED;
 	}
 
 	if (!command_prompt) {
 	    tree_start_search (tree);
 	    tree_do_search (tree, key);
-	    return 1;
+	    return MSG_HANDLED;
 	}
-	return tree->is_panel;
+	return tree->is_panel ? MSG_HANDLED : MSG_NOT_HANDLED;
     }
 
-    return 0;
+    return MSG_NOT_HANDLED;
 }
 
 static void
@@ -1009,58 +966,74 @@ tree_frame (Dlg_head *h, WTree *tree)
     }
 }
 
-
-static int
-tree_callback (WTree *tree, int msg, int par)
+static void
+tree_rescan_command (void *data)
 {
+    WTree *tree = data;
+    tree_rescan_cmd (tree);
+}
+
+static void
+tree_rmdir_command (void *data)
+{
+    WTree *tree = data;
+    tree_rmdir_cmd (tree);
+}
+
+static cb_ret_t
+tree_callback (Widget *w, widget_msg_t msg, int parm)
+{
+    WTree *tree = (WTree *) w;
     Dlg_head *h = tree->widget.parent;
 
     switch (msg) {
     case WIDGET_DRAW:
 	tree_frame (h, tree);
 	show_tree (tree);
-	return 1;
+	return MSG_HANDLED;
 
     case WIDGET_KEY:
-	return tree_key (tree, par);
+	return tree_key (tree, parm);
 
     case WIDGET_FOCUS:
 	tree->active = 1;
-	define_label (h, 1, _("Help"), (voidfn) tree_help_cmd);
-	define_label_data (h, 2, _("Rescan"),
-			   (buttonbarfn) tree_rescan_cmd, tree);
-	define_label_data (h, 3, _("Forget"),
-			   (buttonbarfn) tree_forget_cmd, tree);
-	define_label_data (h, 5, _("Copy"), (buttonbarfn) tree_copy_cmd,
-			   tree);
-	define_label_data (h, 6, _("RenMov"), (buttonbarfn) tree_move_cmd,
-			   tree);
+	buttonbar_set_label (h, 1, _("Help"), tree_help_cmd);
+	buttonbar_set_label_data (h, 2, _("Rescan"),
+	    tree_rescan_command, tree);
+	buttonbar_set_label_data (h, 3, _("Forget"), tree_forget_cmd, tree);
+	buttonbar_set_label_data (h, 5, _("Copy"), tree_copy_cmd, tree);
+	buttonbar_set_label_data (h, 6, _("RenMov"), tree_move_cmd, tree);
 #if 0
 	/* FIXME: mkdir is currently defunct */
-	define_label_data (h, 7, _("Mkdir"), (buttonbarfn) tree_mkdir_cmd,
-			   tree);
+	buttonbar_set_label_data (h, 7, _("Mkdir"), tree_mkdir_cmd, tree);
 #else
-	define_label (h, 7, "", 0);
+	buttonbar_clear_label (h, 7);
 #endif
-	define_label_data (h, 8, _("Rmdir"), (buttonbarfn) tree_rmdir_cmd,
-			   tree);
+	buttonbar_set_label_data (h, 8, _("Rmdir"), tree_rmdir_command, tree);
 	set_navig_label (tree);
-	redraw_labels (h);
+	buttonbar_redraw (h);
 
 
 	/* FIXME: Should find a better way of only displaying the
 	   currently selected item */
 	show_tree (tree);
-	return 1;
+	return MSG_HANDLED;
 
 	/* FIXME: Should find a better way of changing the color of the
 	   selected item */
+
     case WIDGET_UNFOCUS:
 	tree->active = 0;
 	show_tree (tree);
-	return 1;
+	return MSG_HANDLED;
+
+    case WIDGET_DESTROY:
+	tree_destroy (tree);
+	return MSG_HANDLED;
+
+    default:
+	return default_proc (msg, parm);
     }
-    return default_proc (msg, par);
 }
 
 WTree *
@@ -1069,8 +1042,7 @@ tree_new (int is_panel, int y, int x, int lines, int cols)
     WTree *tree = g_new (WTree, 1);
 
     init_widget (&tree->widget, y, x, lines, cols,
-		 (callback_fn) tree_callback, (destroy_fn) tree_destroy,
-		 (mouse_h) event_callback, NULL);
+		 tree_callback, event_callback);
     tree->is_panel = is_panel;
     tree->selected_ptr = 0;
 
