@@ -16,7 +16,7 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #define NO_SYSLOG
@@ -87,7 +87,7 @@ static BOOL cli_send_smb(struct cli_state *cli)
 struct
 {
   int err;
-  char *message;
+  const char *message;
 } const rap_errmap[] =
 {
   {5,    "User has insufficient privilege" },
@@ -190,7 +190,7 @@ static void cli_setup_packet(struct cli_state *cli)
 	}
 }
 
-
+#if 0
 /*****************************************************************************
  Convert a character pointer in a cli_call_api() response to a form we can use.
  This function contains code to prevent core dumps if the server returns 
@@ -213,12 +213,12 @@ static char *fix_char_ptr(unsigned int datap, unsigned int converter,
 		}
 	}
 }
-
+#endif /* 0 */
 /****************************************************************************
   send a SMB trans or trans2 request
   ****************************************************************************/
 static BOOL cli_send_trans(struct cli_state *cli, int trans, 
-                           char *name, int pipe_name_len, 
+                           const char *name, int pipe_name_len, 
                            int fid, int flags,
                            uint16 *setup, int lsetup, int msetup,
                            char *param, int lparam, int mparam,
@@ -422,6 +422,7 @@ static BOOL cli_receive_trans(struct cli_state *cli,int trans,
 	return(True);
 }
 
+#if 0
 /****************************************************************************
 Call a remote api on an arbitrary pipe.  takes param, data and setup buffers.
 ****************************************************************************/
@@ -446,6 +447,7 @@ BOOL cli_api_pipe(struct cli_state *cli, char *pipe_name, int pipe_name_len,
                             rparam, (int *)rparam_count,
                             rdata, (int *)rdata_count));
 }
+#endif /*0 */
 
 /****************************************************************************
 call a remote api
@@ -469,7 +471,7 @@ BOOL cli_api(struct cli_state *cli,
                             rdata, rdrcnt));
 }
 
-
+#if 0
 /****************************************************************************
 perform a NetWkstaUserLogon
 ****************************************************************************/
@@ -531,11 +533,12 @@ BOOL cli_NetWkstaUserLogon(struct cli_state *cli,char *user, char *workstation)
       free(rdata);
 	return (cli->rap_error == 0);
 }
+#endif /*0 */
 
 /****************************************************************************
 call a NetShareEnum - try and browse available connections on a host
 ****************************************************************************/
-BOOL cli_RNetShareEnum(struct cli_state *cli, void (*fn)(const char *, uint32, const char *))
+int cli_RNetShareEnum(struct cli_state *cli, void (*fn)(const char *, uint32, const char *, void *), void * state)
 {
   char *rparam = NULL;
   char *rdata = NULL;
@@ -553,12 +556,16 @@ BOOL cli_RNetShareEnum(struct cli_state *cli, void (*fn)(const char *, uint32, c
   pstrcpy(p,"B13BWz");
   p = skip_string(p,1);
   SSVAL(p,0,1);
-  SSVAL(p,2,0xFFFF);
+  /*
+   * Win2k needs a *smaller* buffer than 0xFFFF here -
+   * it returns "out of server memory" with 0xFFFF !!! JRA.
+   */
+  SSVAL(p,2,0xFFE0);
   p += 4;
 
   if (cli_api(cli, 
               param, PTR_DIFF(p,param), 1024,  /* Param, length, maxlen */
-              NULL, 0, 0xFFFF,            /* data, length, maxlen */
+              NULL, 0, 0xFFE0,            /* data, length, maxlen - Win2k needs a small buffer here too ! */
               &rparam, &rprcnt,                /* return params, length */
               &rdata, &rdrcnt))                /* return data, length */
     {
@@ -574,8 +581,8 @@ BOOL cli_RNetShareEnum(struct cli_state *cli, void (*fn)(const char *, uint32, c
 		      char *sname = p;
 		      int type = SVAL(p,14);
 		      int comment_offset = IVAL(p,16) & 0xFFFF;
-		      char *cmnt = comment_offset?(rdata+comment_offset-converter):"";
-		      fn(sname, type, cmnt);
+		      const char *cmnt = comment_offset?(rdata+comment_offset-converter):"";
+		      fn(sname, type, cmnt, state);
 	      }
       } else {
 	      DEBUG(4,("NetShareEnum res=%d\n", res));
@@ -601,7 +608,7 @@ The callback function takes 3 arguments: the machine name, the server type and
 the comment.
 ****************************************************************************/
 BOOL cli_NetServerEnum(struct cli_state *cli, char *workgroup, uint32 stype,
-		       void (*fn)(const char *, uint32, const char *))
+		       void (*fn)(const char *, uint32, const char *, void *), void *state)
 {
 	char *rparam = NULL;
 	char *rdata = NULL;
@@ -647,12 +654,12 @@ BOOL cli_NetServerEnum(struct cli_state *cli, char *workgroup, uint32 stype,
 			for (i = 0;i < count;i++, p += 26) {
 				char *sname = p;
 				int comment_offset = (IVAL(p,22) & 0xFFFF)-converter;
-				char *cmnt = comment_offset?(rdata+comment_offset):"";
+				const char *cmnt = comment_offset?(rdata+comment_offset):"";
 				if (comment_offset < 0 || comment_offset > rdrcnt) continue;
 
 				stype = IVAL(p,18) & ~SV_TYPE_LOCAL_LIST_ONLY;
 
-				fn(sname, stype, cmnt);
+				fn(sname, stype, cmnt, state);
 			}
 		}
 	}
@@ -670,7 +677,7 @@ BOOL cli_NetServerEnum(struct cli_state *cli, char *workgroup, uint32 stype,
 
 static  struct {
     int prot;
-    char *name;
+    const char *name;
   }
 const prots[] = 
     {
@@ -750,6 +757,7 @@ BOOL cli_session_setup(struct cli_state *cli,
 		p += passlen;
 		pstrcpy(p,user);
 		strupper(p);
+		unix_to_dos (p, True);
 	}
 	else
 	{
@@ -772,6 +780,7 @@ BOOL cli_session_setup(struct cli_state *cli,
 		p += SVAL(cli->outbuf,smb_vwv8);
 		pstrcpy(p,user);
 		strupper(p);
+		unix_to_dos (p, True);
 		p = skip_string(p,1);
 		pstrcpy(p,workgroup);
 		strupper(p);
@@ -816,7 +825,7 @@ BOOL cli_session_setup(struct cli_state *cli,
 /****************************************************************************
  Send a uloggoff.
 *****************************************************************************/
-
+#if 0
 BOOL cli_ulogoff(struct cli_state *cli)
 {
         memset(cli->outbuf,'\0',smb_size);
@@ -832,12 +841,13 @@ BOOL cli_ulogoff(struct cli_state *cli)
 
         return CVAL(cli->inbuf,smb_rcls) == 0;
 }
+#endif /*0 */
 
 /****************************************************************************
 send a tconX
 ****************************************************************************/
 BOOL cli_send_tconX(struct cli_state *cli, 
-		    char *share, char *dev, char *pass, int passlen)
+		    const char *share, const char *dev, const char *pass, int passlen)
 {
 	fstring fullshare, pword;
 	char *p;
@@ -861,6 +871,7 @@ BOOL cli_send_tconX(struct cli_state *cli,
 
 	slprintf(fullshare, sizeof(fullshare)-1,
 		 "\\\\%s\\%s", cli->desthost, share);
+	unix_to_dos(fullshare, True);
 	strupper(fullshare);
 
 	set_message(cli->outbuf,4,
@@ -909,7 +920,7 @@ BOOL cli_send_tconX(struct cli_state *cli,
 	return True;
 }
 
-
+#if 0
 /****************************************************************************
 send a tree disconnect
 ****************************************************************************/
@@ -927,6 +938,7 @@ BOOL cli_tdis(struct cli_state *cli)
 	
 	return CVAL(cli->inbuf,smb_rcls) == 0;
 }
+#endif /*0 */
 
 /****************************************************************************
 rename a file
@@ -1063,8 +1075,7 @@ BOOL cli_rmdir(struct cli_state *cli, char *dname)
 	return True;
 }
 
-
-
+#if 0
 /****************************************************************************
 open a file
 ****************************************************************************/
@@ -1107,7 +1118,7 @@ int cli_nt_create(struct cli_state *cli, char *fname)
 
 	return SVAL(cli->inbuf,smb_vwv2 + 1);
 }
-
+#endif /*0 */
 
 /****************************************************************************
 open a file
@@ -1211,7 +1222,7 @@ BOOL cli_close(struct cli_state *cli, int fnum)
 	return True;
 }
 
-
+#if 0
 /****************************************************************************
   lock a file
 ****************************************************************************/
@@ -1297,7 +1308,7 @@ BOOL cli_unlock(struct cli_state *cli, int fnum, uint32 offset, uint32 len, int
 
 	return True;
 }
-
+#endif /*0 */
 
 
 /****************************************************************************
@@ -1397,7 +1408,7 @@ size_t cli_read(struct cli_state *cli, int fnum, char *buf, off_t offset, size_t
 /****************************************************************************
 issue a single SMBwrite and don't wait for a reply
 ****************************************************************************/
-static void cli_issue_write(struct cli_state *cli, int fnum, off_t offset, uint16 mode, char *buf,
+static void cli_issue_write(struct cli_state *cli, int fnum, off_t offset, uint16 mode, const char *buf,
 			    size_t size, int i)
 {
 	char *p;
@@ -1441,7 +1452,7 @@ static void cli_issue_write(struct cli_state *cli, int fnum, off_t offset, uint1
 ****************************************************************************/
 ssize_t cli_write(struct cli_state *cli,
 		  int fnum, uint16 write_mode,
-		  char *buf, off_t offset, size_t size)
+		  const char *buf, off_t offset, size_t size)
 {
 	int bwritten = 0;
 	int issued = 0;
@@ -1487,12 +1498,12 @@ ssize_t cli_write(struct cli_state *cli,
 	return bwritten;
 }
 
-
+#if 0
 /****************************************************************************
   write to a file using a SMBwrite and not bypassing 0 byte writes
 ****************************************************************************/
 ssize_t cli_smbwrite(struct cli_state *cli,
-		     int fnum, char *buf, off_t offset, size_t size)
+		     int fnum, const char *buf, off_t offset, size_t size)
 {
 	char *p;
 
@@ -1526,7 +1537,7 @@ ssize_t cli_smbwrite(struct cli_state *cli,
 
 	return SVAL(cli->inbuf,smb_vwv0);
 }
-
+#endif /*0 */
 
 /****************************************************************************
 do a SMBgetattrE call
@@ -1663,6 +1674,7 @@ BOOL cli_setatr(struct cli_state *cli, char *fname, uint16 attr, time_t t)
 	return True;
 }
 
+#if 0
 /****************************************************************************
 send a qpathinfo call
 ****************************************************************************/
@@ -1804,7 +1816,7 @@ BOOL cli_qpathinfo2(struct cli_state *cli, const char *fname,
 	if (rparam) free(rparam);
 	return True;
 }
-
+#endif	/* 0 */
 
 /****************************************************************************
 send a qfileinfo call
@@ -1886,7 +1898,7 @@ by NT and 2 is used by OS/2
 ****************************************************************************/
 static int interpret_long_filename(int level,char *p,file_info *finfo)
 {
-	extern file_info def_finfo;
+	extern file_info const def_finfo;
 
 	if (finfo)
 		memcpy(finfo,&def_finfo,sizeof(*finfo));
@@ -1989,7 +2001,7 @@ static int interpret_long_filename(int level,char *p,file_info *finfo)
   do a directory listing, calling fn on each file found
   ****************************************************************************/
 int cli_list(struct cli_state *cli,const char *Mask,uint16 attribute, 
-	     void (*fn)(file_info *, const char *))
+	     void (*fn)(file_info *, const char *, void *), void *state)
 {
 	int max_matches = 512;
 	/* NT uses 260, OS/2 uses 2. Both accept 1. */
@@ -2140,7 +2152,7 @@ int cli_list(struct cli_state *cli,const char *Mask,uint16 attribute,
 
 	for (p=dirlist,i=0;i<total_received;i++) {
 		p += interpret_long_filename(info_level,p,&finfo);
-		fn(&finfo, Mask);
+		fn(&finfo, Mask, state);
 	}
 
 	/* free up the dirlist buffer */
@@ -2149,98 +2161,6 @@ int cli_list(struct cli_state *cli,const char *Mask,uint16 attribute,
 }
 
 
-/****************************************************************************
-Send a SamOEMChangePassword command
-****************************************************************************/
-
-BOOL cli_oem_change_password(struct cli_state *cli, const char *user, const char *new_password,
-                             const char *old_password)
-{
-  char param[16+sizeof(fstring)];
-  char data[532];
-  char *p = param;
-  fstring upper_case_old_pw;
-  fstring upper_case_new_pw;
-  unsigned char old_pw_hash[16];
-  unsigned char new_pw_hash[16];
-  int data_len;
-  int param_len = 0;
-  char *rparam = NULL;
-  char *rdata = NULL;
-  int rprcnt, rdrcnt;
-
-  if (strlen(user) >= sizeof(fstring)-1) {
-    DEBUG(0,("cli_oem_change_password: user name %s is too long.\n", user));
-    return False;
-  }
-
-  SSVAL(p,0,214); /* SamOEMChangePassword command. */
-  p += 2;
-  pstrcpy(p, "zsT");
-  p = skip_string(p,1);
-  pstrcpy(p, "B516B16");
-  p = skip_string(p,1);
-  pstrcpy(p,user);
-  p = skip_string(p,1);
-  SSVAL(p,0,532);
-  p += 2;
-
-  param_len = PTR_DIFF(p,param);
-
-  /*
-   * Get the Lanman hash of the old password, we
-   * use this as the key to make_oem_passwd_hash().
-   */
-  memset(upper_case_old_pw, '\0', sizeof(upper_case_old_pw));
-  fstrcpy(upper_case_old_pw, old_password);
-  strupper(upper_case_old_pw);
-  E_P16((uchar *)upper_case_old_pw, old_pw_hash);
-
-	if (!make_oem_passwd_hash( data, new_password, old_pw_hash, False))
-	{
-		return False;
-	}
-
-  /* 
-   * Now place the old password hash in the data.
-   */
-  memset(upper_case_new_pw, '\0', sizeof(upper_case_new_pw));
-  fstrcpy(upper_case_new_pw, new_password);
-  strupper(upper_case_new_pw);
-
-  E_P16((uchar *)upper_case_new_pw, new_pw_hash);
-
-  E_old_pw_hash( new_pw_hash, old_pw_hash, (uchar *)&data[516]);
-
-  data_len = 532;
-    
-  if (cli_send_trans(cli,SMBtrans,
-                    PIPE_LANMAN,strlen(PIPE_LANMAN),      /* name, length */
-                    0,0,                                  /* fid, flags */
-                    NULL,0,0,                             /* setup, length, max */
-                    param,param_len,2,                    /* param, length, max */
-                    data,data_len,0                       /* data, length, max */
-                   ) == False) {
-    DEBUG(0,("cli_oem_change_password: Failed to send password change for user %s\n",
-              user ));
-    return False;
-  }
-
-  if (cli_receive_trans(cli,SMBtrans,
-                       &rparam, &rprcnt,
-                       &rdata, &rdrcnt)) {
-    if (rparam)
-      cli->rap_error = SVAL(rparam,0);
-  }
-
-  if (rparam)
-    free(rparam);
-  if (rdata)
-    free(rdata);
-
-  return (cli->rap_error == 0);
-}
-
 /****************************************************************************
 send a negprot command
 ****************************************************************************/
@@ -2778,115 +2698,6 @@ BOOL cli_establish_connection(struct cli_state *cli,
 }
 
 
-/****************************************************************************
-  cancel a print job
-  ****************************************************************************/
-int cli_printjob_del(struct cli_state *cli, int job)
-{
-	char *rparam = NULL;
-	char *rdata = NULL;
-	char *p;
-	int rdrcnt,rprcnt, ret = -1;
-	pstring param;
-
-	memset(param,'\0',sizeof(param));
-
-	p = param;
-	SSVAL(p,0,81);		/* DosPrintJobDel() */
-	p += 2;
-	pstrcpy(p,"W");
-	p = skip_string(p,1);
-	pstrcpy(p,"");
-	p = skip_string(p,1);
-	SSVAL(p,0,job);     
-	p += 2;
-	
-	if (cli_api(cli, 
-		    param, PTR_DIFF(p,param), 1024,  /* Param, length, maxlen */
-		    NULL, 0, CLI_BUFFER_SIZE,            /* data, length, maxlen */
-		    &rparam, &rprcnt,                /* return params, length */
-		    &rdata, &rdrcnt)) {               /* return data, length */
-		ret = SVAL(rparam,0);
-	}
-
-	if (rparam) free(rparam);
-	if (rdata) free(rdata);
-
-	return ret;
-}
-
-
-/****************************************************************************
-call fn() on each entry in a print queue
-****************************************************************************/
-int cli_print_queue(struct cli_state *cli, 
-		    void (*fn)(struct print_job_info *))
-{
-	char *rparam = NULL;
-	char *rdata = NULL;
-	char *p;
-	int rdrcnt, rprcnt;
-	pstring param;
-	int result_code=0;
-	int i = -1;
-	
-	memset(param,'\0',sizeof(param));
-
-	p = param;
-	SSVAL(p,0,76);         /* API function number 76 (DosPrintJobEnum) */
-	p += 2;
-	pstrcpy(p,"zWrLeh");   /* parameter description? */
-	p = skip_string(p,1);
-	pstrcpy(p,"WWzWWDDzz");  /* returned data format */
-	p = skip_string(p,1);
-	pstrcpy(p,cli->share);    /* name of queue */
-	p = skip_string(p,1);
-	SSVAL(p,0,2);   /* API function level 2, PRJINFO_2 data structure */
-	SSVAL(p,2,1000); /* size of bytes of returned data buffer */
-	p += 4;
-	pstrcpy(p,"");   /* subformat */
-	p = skip_string(p,1);
-
-	DEBUG(4,("doing cli_print_queue for %s\n", cli->share));
-
-	if (cli_api(cli, 
-		    param, PTR_DIFF(p,param), 1024,  /* Param, length, maxlen */
-		    NULL, 0, CLI_BUFFER_SIZE,            /* data, length, maxlen */
-		    &rparam, &rprcnt,                /* return params, length */
-		    &rdata, &rdrcnt)) {               /* return data, length */
-		int converter;
-		result_code = SVAL(rparam,0);
-		converter = SVAL(rparam,2);       /* conversion factor */
-
-		if (result_code == 0) {
-			struct print_job_info job;
-			
-			p = rdata; 
-
-			for (i = 0; i < SVAL(rparam,4); ++i) {
-				job.id = SVAL(p,0);
-				job.priority = SVAL(p,2);
-				fstrcpy(job.user,
-					fix_char_ptr(SVAL(p,4), converter, 
-						     rdata, rdrcnt));
-				job.t = make_unix_date3(p + 12);
-				job.size = IVAL(p,16);
-				fstrcpy(job.name,fix_char_ptr(SVAL(p,24), 
-							      converter, 
-							      rdata, rdrcnt));
-				fn(&job);				
-				p += 28;
-			}
-		}
-	}
-
-	/* If any parameters or data were returned, free the storage. */
-	if(rparam) free(rparam);
-	if(rdata) free(rdata);
-
-	return i;
-}
-
 /****************************************************************************
 check for existance of a dir
 ****************************************************************************/
@@ -2895,7 +2706,7 @@ BOOL cli_chkpath(struct cli_state *cli, char *path)
 	pstring path2;
 	char *p;
 	
-	safe_strcpy(path2,path,sizeof(pstring));
+	safe_strcpy(path2,path,sizeof(pstring)-1);
 	trim_string(path2,NULL,"\\");
 	if (!*path2) *path2 = '\\';
 	
@@ -2918,7 +2729,7 @@ BOOL cli_chkpath(struct cli_state *cli, char *path)
 	return True;
 }
 
-
+#if 0
 /****************************************************************************
 start a message sequence
 ****************************************************************************/
@@ -3012,8 +2823,9 @@ BOOL cli_message_end(struct cli_state *cli, int grp)
 
 	return True;
 }      
+#endif /*0 */
 
-
+#if 0	/* May be useful one day */
 /****************************************************************************
 query disk space
 ****************************************************************************/
@@ -3036,3 +2848,4 @@ BOOL cli_dskattr(struct cli_state *cli, int *bsize, int *total, int *avail)
 	
 	return True;
 }
+#endif	/* 0 */
